
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CodecG711U_cxx_Version =
    "$Id: CodecG711U.cxx,v 1.1 2002/12/09 22:10:25 sprajpat Exp $";

#include "global.h"
#include <cassert>

#include "CodecG711U.hxx"
#include "codec/g711.h"
#include "cpLog.h"

using namespace Vocal::UA;

int 
CodecG711U::encode(char* data, int length, char* encBuf, int &encodedLength)
{
    cpLog(LOG_DEBUG_STACK,"CodecG711U::encode: %d", length);
    if(encodedLength < length/2)
    {
       cpLog(LOG_ERR, "Not enough space to put encoded data");
       return -1; 
    }
    short* srcData = reinterpret_cast<short*>(data);
    for(int i = 0; i < length/2; i++)
    {
        encBuf[i] = linear2ulaw(srcData[i]);
    }
    encodedLength = length/2;
    return (0);
}
 
int 
CodecG711U::decode(char* data, int length, char* decBuf, int& decodedLength)
{
    cpLog(LOG_DEBUG_STACK,"CodecG711U::decode: %d", length);
    if(decodedLength < length * 2)
    {
       cpLog(LOG_ERR, "Not enough space to put decoded data");
       return -1; 
    }
    short * retData = reinterpret_cast<short*>(decBuf);
    char* srcData = (data);
    for(int i = 0; i < length; i++)
    {
        retData[i] = ulaw2linear(srcData[i]);
    }
    decodedLength = length *2;
    return (0);
}
