#ifndef CodecTelEvent_hxx
#define CodecTelEvent_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CodecTelEvent_hxx_Version = 
    "$Id: CodecTelEvent.hxx,v 1.1.2.1 2003/01/28 02:16:56 sprajpat Exp $";

#include "CodecAdaptor.hxx"

namespace Vocal
{

namespace UA
{

/** CodecTelEvent - placeholder implementation for offering/answering
    DTMF TONE event type 100.
 */
class CodecTelEvent  : public CodecAdaptor
{
    public:
      /**Constructor to create a CodecAdaptor, the priority indicate
       * the pref when multiple codecs can be used, a 0 priority means
       * equal preferrence
       */
      CodecTelEvent(int priority=0)
         : CodecAdaptor(DTMF_TONE, priority) 
      { 
          myEncodingName = "telephone-event";
          myClockRate = 8000;
          myMediaType = AUDIO;
          myAttrValueMap["fmtp"] = "100 0-15";
      };

      ///Form raw data to  codec type 
      int decode(char* data, int length, char* decBuf, int &decodedLength);

      ///from codec type to raw data 
      int encode(char* data, int length, char* encBuf, int& encodedLength);

      /// Virtual destructor
      ~CodecTelEvent() { };

      ///
      virtual string className() { return "CodecTelEvent"; }

   protected:
      /** Suppress copying
      */
      CodecTelEvent(const CodecTelEvent &);
        
      /** Suppress copying
      */
      const CodecTelEvent & operator=(const CodecTelEvent &);
};

}
}

#endif
