/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSReader_cxx_Version =
    "$Id: PSReader.cxx,v 1.14 2002/09/26 01:14:02 bko Exp $";

#include <sys/types.h>
#include <sys/errno.h>
#include <stdio.h>
#include <unistd.h>

#include "PSReader.hxx"
#include "VNetworkException.hxx"
#include "NetworkAddress.h"
#include "PSNetworkClient.hxx"
#include "Connection.hxx"
#include "cpLog.h"
#include "Vpp_def.h"
#include "PSSecret.hxx"

PSReader::PSReader(const NetworkAddress& netAddress, Data password)
        : _netAddress(netAddress),
	  myPassword(password),
        _clientSocket()
{
}


string
PSReader::getItem(const string& group, const string &name) 
    throw (VNetworkException&)
{
    cpLog(LOG_DEBUG, "PSReader::PSReader - reading item: (%s,%s)",
          group.c_str(), name.c_str());

    Connection& conn = getConnection();

    char buf[256];
    
    sprintf(buf, "%s %s %s %s\n\n", 
	    GET_REQ,
	    group.c_str(), 
	    name.c_str(),
	    VPP_VERSION
	);

    Data result;
    int length;
    Data data;

    sendCommand(conn, buf, result, length, data, true);

    string dRead;

/*    if(result == "200 OK\n")
    {
*/
    cpLog(LOG_DEBUG, "Data read: (%s)", dRead.c_str());
    LocalScopeAllocator lo;
    dRead = data.getData(lo);
    return dRead;
}


void
PSReader::getList(const string &group, vector < Data > & retList)
throw (VNetworkException&)
{
    cpLog(LOG_DEBUG, "PSReader::PSReader - reading group: (%s)",
          group.c_str());

    Connection& conn = getConnection();

    char buf[256];
    sprintf(buf, "%s %s %s\n\n", 
	    NLIST_REQ, 
	    group.c_str(), 
	    VPP_VERSION
	);

    Data result;
    int length;
    Data data;
	
    sendCommand(conn, buf, result, length, data, false);

    string dRead;

    if(result == "200 OK\n")
    {
        cpLog(LOG_DEBUG, "Data read: (%s)", dRead.c_str());
        parseTheItemList(data, retList);
    }
}


void
PSReader::parseTheItemList(Data buf, vector < Data > & retList)
{
    bool matchFail = false;
    Data tok = buf.parse(",\n ", &matchFail);

    while (!matchFail)
    {
        retList.push_back(tok);
        tok = buf.parse(",\n ", &matchFail);
    }

    if(matchFail)
    {
        retList.push_back(buf);
        buf.erase();
    }
}


PSReader::~PSReader()
{
}


Connection&
PSReader::getConnection() throw (VNetworkException&)
{

    _clientSocket = new PSNetworkClient(_netAddress, true, false);
    return (_clientSocket->getConn(PSSecret::readSecret()));

}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
