/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LockedConnection_hxx_Version =
    "$Id: LockedConnection.hxx,v 1.10 2002/09/26 01:14:03 bko Exp $";

#include "TlsConnection.hxx"

/** Class provides a wrapper around a the connection object with a being
    serviced flag to ensure that only one thread tries to service a connection
    at a give time
*/
class LockedConnection : public TlsConnection
{

public:
    
	/* Create a locked connection, optionally specifying if the
	   connections are blocking (default to true) */
	LockedConnection (bool blocking = true)
	    : _serviced(false),
	      myReadAuthorized(false),
	      myWriteAuthorized(false)
        {
        }

	LockedConnection(const LockedConnection& src)
		:TlsConnection(src),
		 _serviced(src._serviced),
		 myReadAuthorized(src.myReadAuthorized),
		 myWriteAuthorized(src.myWriteAuthorized)
	{
	}

	///
        ~LockedConnection ()
        {
	}

	/// Returns true if the connection is not being serviced
        bool notServiced() const
        {
            return !(_serviced);
        };

	/// Sets a connection to being serviced
        void beingServiced()
        {
            _serviced = true;
        };

	/// Sets a connection to not being serviced
        void doneService()
        {
            _serviced = false;
        };

	/// true if this connection has been authorized for reading
	bool isReadAuthorized() const
	{
	    return myReadAuthorized || myWriteAuthorized;
	}

	/// true if this connection has been authorized for writing
	bool isWriteAuthorized() const
	{
	    return myWriteAuthorized;
	}

	/// set myReadAuthorized to true
	void readAuthorize()
	{
	    myReadAuthorized = true;
	}

	/// set myWriteAuthorized to true
	void writeAuthorize()
	{
	    myWriteAuthorized = true;
	}

        /// assignment operator
	const LockedConnection& operator=(const LockedConnection& x)
	{
		if(&x != this)
		{
			this->TlsConnection::operator=(x);
			_serviced = x._serviced;
			myReadAuthorized = x.myReadAuthorized;
			myWriteAuthorized = x.myWriteAuthorized;
		}
		return *this;
	}

    private:

	/// boolean flag to trag if this connection is being serviced
        bool _serviced;

	/// boolean flag set when read authentication available
	bool myReadAuthorized;

	/// boolean flag set when write authentication available
	bool myWriteAuthorized;
};


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
