/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CallAgent_cxx_Version =
    "$Id: CallAgent.cxx,v 1.21.2.5 2003/03/13 00:27:42 sprajpat Exp $";

#include "ByeMsg.hxx"
#include "InviteMsg.hxx"
#include "CancelMsg.hxx"
#include "SystemInfo.hxx"
#include "ControlStateFactory.hxx"
#include "UaCallControl.hxx"
#include "UaServer.hxx"
#include "UaFacade.hxx"
#include "SdpHandler.hxx"
#include "CallDB.hxx"
#include "UaClient.hxx"
#include "CallAgent.hxx"
#include "UaConfiguration.hxx"
#include "SipServer.hxx"
#include "SipDiversion.hxx"
#include "Authenticate.hxx"
#include "FileMediaDevice.hxx"
#include "UaCommandLine.hxx"
#ifdef USE_VM
#include "VmcpDevice.hxx"
#endif
#ifdef USE_MPEGLIB
#include "VideoDevice.hxx"
#endif

#include "MediaController.hxx"

using namespace Vocal;
using namespace Vocal::UA;

long CallAgent::myRotatingId = 0;

CallAgent::CallAgent(Sptr<SipMsg> sipMsg, AgentRole aRole)
   : BasicAgent(myRotatingId++) , myRole(aRole), myActiveFlg(true)
{ 

    myState = ControlStateFactory::instance().getState(INIT);

    ///Create new UserAgent
    if(aRole == A_CLIENT)
    {
        //Create the invite message
        myInvokee = new UaClient(sipMsg, UaFacade::instance().getSipTransceiver(), this);
        myState->makeCall(*this);
    }
    else
    {
        myInvokee = new UaServer(sipMsg, UaFacade::instance().getSipTransceiver(), this);
        myInvokee->receivedMsg(sipMsg);
        myState->ringing(*this);
        myActiveFlg = true;
    }

    //Add the callleg in CallDb
    CallDB::instance().addCallLeg(myInvokee);

};
Sptr<StatusMsg>
CallAgent::doholdresume200OKstuff(const Sptr<SipMsg>& msg, SdpSession& remoteSdp )
{
   Sptr<StatusMsg> statusMsg;
   Sptr<SipCommand> sCommand;
   sCommand.dynamicCast(myInvokee->getRequest());
   assert(sCommand != 0);
   statusMsg = new StatusMsg(*sCommand, 200);
   // Set Contact header
   Sptr< SipUrl > myUrl = new SipUrl;
   myUrl->setUserValue( UaConfiguration::instance().getValue(UserNameTag) );
   myUrl->setHost(theSystem.gethostAddress());
   myUrl->setPort( UaConfiguration::instance().getValue(LocalSipPortTag) );
   SipContact myContact;
   myContact.setUrl( myUrl );
   statusMsg->setNumContact( 0 );    // Clear old contact
   statusMsg->setContact( myContact );
   SipServer lServer("Vovida-SIP-SoftPhone/0.1");
   statusMsg->setServer(lServer);
   Data S_host;
   if((UaConfiguration::instance().getValue(NATAddressIPTag)).length())
   {
      S_host = UaConfiguration::instance().getValue(NATAddressIPTag);
   }
   else
   {
      S_host = theSystem.gethostAddress();
   }

   SipSdp localSdp;
   int sessionId =
      myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
   SdpSession localS =
      MediaController::instance().getSdp(sessionId, remoteSdp);
   setHost(localS, S_host);
   localSdp.setSdpDescriptor(localS);
   statusMsg->setContentData(&localSdp, 0);
   return statusMsg;
}

void 
CallAgent::placeCall()
{
    cpLog(LOG_DEBUG, "CallAgent::placeCall()");
    Sptr<SipMsg> sipMsg = myInvokee->getRequest();
    myInvokee->sendMsg(sipMsg);
    UaFacade::instance().postInfo(sipMsg);
}


void 
CallAgent::doBye()
{
    if(!myState->canBye()){
	return;
    }
    cpLog(LOG_DEBUG, "CallAgent::doBye()");
    Sptr<SipMsg> bMsg = myInvokee->sendBye();
    if(bMsg != 0) UaFacade::instance().postInfo(bMsg);
    endCall();
    strstream str;
    str << "L_HANGUP " << ends;
    UaFacade::instance().postMsg(str.str());
    str.freeze(false);
}

void 
CallAgent::endCall()
{
    if(!myActiveFlg)
    {
        cpLog(LOG_INFO, "Call Agent no more active, ignoring endCall()");
        return;
    }

    cpLog(LOG_DEBUG, "CallAgent::endCall()");
    try{
        int sessionId = myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
        MediaController::instance().freeSession(sessionId);	
	UaFacade::instance().releaseMediaDevice(getId());

	///Transit the controller state
	myState->bye(*this);  //-> TEAR_DOWN
	myState->end(*this);  //-> INIT
    }
    catch(CInvalidStateException& e)
    {
        cpLog(LOG_ERR, "Unexpacted Error ? :(:%s", e.getDescription().c_str());
    }
}

void 
CallAgent::doCancel()
{
    cpLog(LOG_DEBUG, "CallAgent::doCancel()");
    myState->cancel(*this);
    myState->end(*this);
}

void
CallAgent::inCall()
{
    cpLog(LOG_DEBUG, "CallAgent::inCall()");
    myState->inCall(*this);

#ifdef USE_MPEGLIB
    int video  = atoi(UaConfiguration::instance().getValue(VideoTag).c_str());
    //Even if video mode is on, when running in command line mode force the
    //non-video option
    if(video && (!UaCommandLine::instance()->getIntOpt("cmdline")))
    {
        Sptr<VideoDevice> vDevice;
        vDevice.dynamicCast(UaFacade::instance().getMediaDevice(getId()));
        assert(vDevice != 0);
        vDevice->startMedia(myInvokee->getRemoteSdp()->getSdpDescriptor());
    }
    else
    {
        startSession(myInvokee->getLocalSdp()->getSdpDescriptor(), 
                 myInvokee->getRemoteSdp()->getSdpDescriptor());
    }
#else
    startSession(myInvokee->getLocalSdp()->getSdpDescriptor(), 
                 myInvokee->getRemoteSdp()->getSdpDescriptor());
#endif
}

void
CallAgent::callFailed()
{
    cpLog(LOG_DEBUG, "CallAgent::callFailed()");
    myState->cancel(*this);
    myState->end(*this);
    strstream str;
    str << "L_HANGUP " << ends;
    UaFacade::instance().postMsg(str.str());
    str.freeze(false);
}

///
void 
CallAgent::receivedRequest(UaBase& agent, const Sptr<SipMsg>& msg)
{
    //Notify GUI
    UaFacade::instance().postMsg(msg);
    myState->recvReq(*this, msg);
}
///
void 
CallAgent::receivedStatus(UaBase& agent, const Sptr<SipMsg>& msg)
{
    try
    {
       myState->recvStatus(*this, msg);
       Sptr<StatusMsg> statusMsg;
       statusMsg.dynamicCast(msg);
       assert(statusMsg != 0);
       if(statusMsg->getStatusLine().getStatusCode() == 200 &&
          statusMsg->getCSeq().getMethod() == INVITE_METHOD )
       {
          agent.ackStatus(msg);
       }

       if(statusMsg->getStatusLine().getStatusCode() == 302)
       {
          Sptr<InviteMsg> inviteMsg;
          inviteMsg.dynamicCast(myInvokee->getRequest());
          SipRequestLine& reqLine = inviteMsg->getMutableRequestLine();
          SipContactList contactList = statusMsg->getContactList();
          SipContactList::iterator iter = contactList.begin();
          Sptr < SipUrl > iterSipUrl;
          iterSipUrl.dynamicCast( ( *iter )->getUrl() );
          reqLine.setUrl(iterSipUrl);
          SipCSeq localseq = agent.getLocalCSeq();
          localseq.incrCSeq();
          agent.setLocalCSeq(localseq);
          inviteMsg->setCSeq(localseq);
          agent.getSipTransceiver()->sendAsync(inviteMsg);
       }


       //Notify GUI
       UaFacade::instance().postMsg(msg);
       if (msg->getCSeq().getMethod() == CANCEL_METHOD)
       {
          UaFacade::instance().postMsg("L_HANGUP ");
       }
    }
    catch(CInvalidStateException& e)
    {
       cpLog(LOG_ERR, "Unexpacted status:%s", e.getDescription().c_str());
    }
}

void
CallAgent::sendCancel()
{
    Sptr<SipCommand> sCommand;
    sCommand.dynamicCast(myInvokee->getRequest());
    Sptr<CancelMsg> cMsg = new CancelMsg(*sCommand);
    myInvokee->sendMsg(cMsg);
    UaFacade::instance().postInfo(cMsg);
}

void
CallAgent::acceptCall()
{
    //Can only accept call when I am server
    if(myRole != A_SERVER) return;
    try 
    {
        myState->acceptCall(*this);
        Sptr<StatusMsg> statusMsg;
        Sptr<SipCommand> sCommand;
        sCommand.dynamicCast(myInvokee->getRequest());
        assert(sCommand != 0);
        statusMsg = new StatusMsg(*sCommand, 200);

        // Set Contact header
        Sptr< SipUrl > myUrl = new SipUrl;
        myUrl->setUserValue( UaConfiguration::instance().getValue(UserNameTag) );
        myUrl->setHost(theSystem.gethostAddress());
        myUrl->setPort( UaConfiguration::instance().getValue(LocalSipPortTag) );
        SipContact myContact;
        myContact.setUrl( myUrl );
        statusMsg->setNumContact( 0 );    // Clear old contact
        statusMsg->setContact( myContact );

        SipServer lServer("Vovida-SIP-SoftPhone/0.1");
        statusMsg->setServer(lServer);

        Sptr<SipSdp> remoteSdp;
        remoteSdp.dynamicCast(sCommand->getContentData(0));

	if(remoteSdp == 0)
	{
	    // do something safe
	    return;
	}

        assert(remoteSdp != 0);
        Data S_host;
        if((UaConfiguration::instance().getValue(NATAddressIPTag)).length())
        {
           S_host = UaConfiguration::instance().getValue(NATAddressIPTag);

        }
        else
        {
           S_host = theSystem.gethostAddress();

        }

        //Create a session
#ifdef USE_MPEGLIB
        SipSdp localSdp;
        SdpSession localS = MediaController::instance().createSession();
        localSdp.setSdpDescriptor(localS);
        int video  = atoi(UaConfiguration::instance().getValue(VideoTag).c_str());
        //Even if video mode is on, when running in command line mode force the
        //non-video option
        if(video && (!UaCommandLine::instance()->getIntOpt("cmdline")))
        {
            //Offer Video also, Would be nice to move to the Libmedia library when the
            //library integrate the MPEG4 codec. For now encoding/decoding happens
            //deep down in MPEG4IP
            Sptr<VideoDevice> vDevice;
            vDevice.dynamicCast(UaFacade::instance().getMediaDevice(getId()));
            assert(vDevice != 0);
            vDevice->setupSession(remoteSdp->getSdpDescriptor(),
                              localSdp.getSdpDescriptor());
        }
#else
        SipSdp localSdp;
        SdpSession localS = MediaController::instance().createSession(remoteSdp->getSdpDescriptor());
        setHost(localS, S_host);
        localSdp.setSdpDescriptor(localS);
#endif

        statusMsg->setContentData(&localSdp, 0);
        myInvokee->sendMsg(statusMsg);
        UaFacade::instance().postInfo(statusMsg);
    }
    catch(CInvalidStateException& e)
    {
        cpLog(LOG_ERR, "Invalid state transition:%s", e.getDescription().c_str());
    }
}

void
CallAgent::stopCall()
{
    myState->end(*this);
}

void
CallAgent::setDeleted()
{
    myActiveFlg = false;
    cpLog(LOG_DEBUG, "CallAgent::setDeleted:%d", getId());
    CallDB::instance().removeCallData(*myInvokee);
    UaCallControl::instance().removeAgent(getId(), 1000);
}

void
CallAgent::sendBusy()
{
    Sptr<SipCommand> sCommand;
    sCommand.dynamicCast(myInvokee->getRequest());
    Sptr<StatusMsg> sMsg = new StatusMsg(*sCommand, 486);
    myInvokee->sendMsg(sMsg);
    UaFacade::instance().postInfo(sMsg);
    myState->cancel(*this);  
    myState->end(*this);  
}
void
CallAgent::doAuthentication(Sptr<StatusMsg> sMsg)
{
    //User has been challenged, send the inviate with Authorization header
    Data user = UaConfiguration::instance().getValue(UserNameTag);
    Data password = UaConfiguration::instance().getValue(PasswordTag);
    Sptr<SipCommand> sCommand;
    sCommand.dynamicCast(myInvokee->getRequest());
    SipCSeq cSeq = sCommand->getCSeq();
    cSeq.incrCSeq();
    sCommand->setCSeq(cSeq);

    assert(sCommand != 0);
    authenticateMessage(*sMsg,*sCommand, user, password);
    placeCall();
}
void
CallAgent::processHold()
{
    cpLog(LOG_DEBUG, "in CallAgent::processHold this should be invoke in UAC Hold");
    Sptr<InviteMsg> inviteMsg;
    inviteMsg.dynamicCast(myInvokee->getRequest());
    SipFrom fom = inviteMsg->getFrom();
    SipContact cont = inviteMsg->getContact(0);
    Sptr<BaseUrl> cUrl = cont.getUrl();
    Sptr<SipUrl> mcUrl;
    mcUrl.dynamicCast(cUrl);
    SipTo to = inviteMsg->getTo();
    cpLog(LOG_DEBUG, "CallAgent::processHold() Contact host is %s",mcUrl->getHost().logData());
    cpLog(LOG_DEBUG, "CallAgent::processHold() System host is %s",theSystem.gethostAddress());
    if(mcUrl->getHost() != theSystem.gethostAddress()){
	SipFrom from = inviteMsg->getFrom();
	from.setDisplayName("");
	Sptr<BaseUrl> bUrl = to.getUrl();
	Sptr<SipUrl> mUrl;
	mUrl.dynamicCast(bUrl);
	assert(mUrl != 0);
	cpLog(LOG_DEBUG, "CallAgent::processHold() From url  is %s", mUrl->encode().logData());
	from.setUrl(mUrl);
	inviteMsg->setFrom(from);

	// To
	
	Sptr<BaseUrl> bUrl1 = fom.getUrl();
	to.setUser(fom.getUser());
	to.setDisplayName(fom.getDisplayName());
	Sptr<SipUrl> mUrl1;
	mUrl1.dynamicCast(bUrl1);
	assert(mUrl1 != 0);
	to.setUrl(mUrl1);
	inviteMsg->setTo(to);
	cpLog(LOG_DEBUG, "CallAgent::processHold() To  is %s", to.encode().logData());
	cpLog(LOG_DEBUG, "CallAgent::processHold() From is %s", from.encode().logData());
	
    } else {
	//do nothing since We are UAC before, we don't need to flip 
	// TO & FROM
	
    }
//    SipCSeq localseq = myInvokee->getLocalCSeq();
//    localseq.incrCSeq();
//    myInvokee->setLocalCSeq(localseq);
//    inviteMsg->setCSeq(localseq);

    Sptr<SipSdp> sipSdp;
    sipSdp.dynamicCast ( inviteMsg->getContentData( 0 ) );
    assert(sipSdp != 0);

    unsigned int sId = 
         myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
    SdpSession localSdp = MediaController::instance().getSdp(sId, VSDP_SND);
    sipSdp->setSdpDescriptor(localSdp);

    cpLog(LOG_DEBUG, "CallAgent::processHold() is Sending re-invite %s",inviteMsg->encode().logData());
    myInvokee->sendMsg(inviteMsg);
    
}
void
CallAgent::processResume()
{
    cpLog(LOG_DEBUG, "CallAgent::processResume() this should be invoke in UAC Resume ");
    Sptr<InviteMsg> inviteMsg;
    inviteMsg.dynamicCast(myInvokee->getRequest());
//    SipCSeq localseq = myInvokee->getLocalCSeq();
//    localseq.incrCSeq();
//    myInvokee->setLocalCSeq(localseq);
//    inviteMsg->setCSeq(localseq);

    Sptr<SipSdp> sipSdp;
    sipSdp.dynamicCast ( inviteMsg->getContentData( 0 ) );
    assert(sipSdp != 0);
    unsigned int sId = 
         myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
    SdpSession localSdp = MediaController::instance().getSdp(sId);
    sipSdp->setSdpDescriptor(localSdp);

    cpLog(LOG_DEBUG, "Sending InviteMsg %s",inviteMsg->encode().logData() );
    myInvokee->sendMsg(inviteMsg);
    //placeCall();
    
}

void
CallAgent::doResume(Sptr<SipMsg>& msg)
{
    cpLog(LOG_DEBUG, "CallAgent::doResume() this should be invoke in UAC Resume ");
    cpLog(LOG_DEBUG, "CallAgent::processHold() Response is %s",msg->encode().logData());
    myState->inCall(*this);
    Sptr<StatusMsg> smsg;
    smsg.dynamicCast(msg);
    cpLog(LOG_DEBUG, "CallAgent::processHold() is Sending re-invite %s",msg->encode().logData());
    Sptr<SipSdp> remoteSdp;
    remoteSdp.dynamicCast(smsg->getContentData(0));
    myInvokee->setRemoteSdp(remoteSdp);
    assert(remoteSdp != 0);

    unsigned int sId = 
         myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
    MediaController::instance().resumeSession(sId, remoteSdp->getSdpDescriptor());
}
void
CallAgent::dohold()
{
    cpLog(LOG_DEBUG, "CallAgent::dohold() this should be invoke in UAC Hold");
    //UAC just stop the Media since recevied 200 OK for RE-INVITE
    unsigned int sId = myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
    MediaController::instance().suspendSession(sId);
    myState->inHold(*this);
}

void
CallAgent::requestResume(Sptr<SipMsg>& msg)
{
    try{
       Sptr<SipCommand> sCommand;
       sCommand.dynamicCast(myInvokee->getRequest());
       assert(sCommand != 0);
       Sptr<SipSdp> remoteSdp;
       remoteSdp.dynamicCast(sCommand->getContentData(0));
       assert(remoteSdp != 0);
       Sptr<StatusMsg> statusMsg = doholdresume200OKstuff(msg, remoteSdp->getSdpDescriptor());
       myInvokee->sendMsg(statusMsg);
       int sessionId =
          myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
       MediaController::instance().resumeSession(sessionId, remoteSdp->getSdpDescriptor());
       myState->inCall(*this);
    }
    catch(CInvalidStateException& e)
    {
       cpLog(LOG_ERR, "Invalid state transition:%s", e.getDescription().c_str());
    }
}
void
CallAgent::reqResume(Sptr<SipMsg>& msg)
{
    cpLog(LOG_DEBUG, "CallAgent::reqResume() this should be invoke in UAS Resume");
    try
    {
       Sptr<SipCommand> sCommand;
       sCommand.dynamicCast(myInvokee->getRequest());
       assert(sCommand != 0);
       Sptr<SipSdp> remoteSdp;
       remoteSdp.dynamicCast(sCommand->getContentData(0));
       assert(remoteSdp != 0);
       Sptr<StatusMsg> statusMsg = doholdresume200OKstuff(msg,
                                                          remoteSdp->getSdpDescriptor());
       myInvokee->sendMsg(statusMsg);
       int sessionId =
          myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();
       MediaController::instance().resumeSession(sessionId, remoteSdp->getSdpDescriptor());
       strstream str;
        str << "INCALL " << ends;
        UaFacade::instance().postMsg(str.str());
       myState->inCall(*this);
    }
    catch(CInvalidStateException& e)
    {
       cpLog(LOG_ERR, "Invalid state transition:%s", e.getDescription().c_str());
    }
}
void
CallAgent::hold(UaBase& agent, const Sptr<SipMsg>& msg)
{
     cpLog(LOG_DEBUG, "CallAgent::hold() this should be invoke in UAS Hold");

     try 
     {
        Sptr<SipCommand> sCommand;
        sCommand.dynamicCast(myInvokee->getRequest());
        assert(sCommand != 0);
        Sptr<SipSdp> remoteSdp;
        remoteSdp.dynamicCast(sCommand->getContentData(0));
        assert(remoteSdp != 0);
        Sptr<StatusMsg> statusMsg = doholdresume200OKstuff(msg,
                                                           remoteSdp->getSdpDescriptor());
        myInvokee->sendMsg(statusMsg);
   
        unsigned int sId = myInvokee->getLocalSdp()->getSdpDescriptor().getSessionId();	
	MediaController::instance().suspendSession(sId);
        strstream str;
        str << "R_HOLD " << ends;
        UaFacade::instance().postMsg(str.str());
        UaFacade::instance().postInfo(statusMsg);
	myState->inHold(*this);
     }

     catch(CInvalidStateException& e)
     {
        cpLog(LOG_ERR, "Invalid state transition:%s", e.getDescription().c_str());
     }
}

void
CallAgent::startSession(SdpSession& localSdp, SdpSession& remoteSdp)
{
    Sptr<MediaDevice> mDevice;
    unsigned sessionId = localSdp.getSessionId();

    MediaController::instance().addToSession(localSdp, remoteSdp);

    mDevice = UaFacade::instance().getMediaDevice(getId()); 
    MediaController::instance().addDeviceToSession(sessionId, mDevice);

    if(UaFacade::instance().getMode() == CALL_MODE_ANNON)
    {
        Sptr<FileMediaDevice> mFDevice;
        mFDevice.dynamicCast(mDevice);
        string toUser = myInvokee->getRequest()->getTo().getUser().logData();
        toUser += ".wav";
        mFDevice->setFileToPlay(toUser);
        MediaController::instance().startSession(sessionId, VSDP_SND);
    }
    else if(UaFacade::instance().getMode() == CALL_MODE_VMAIL)
    {
#ifdef USE_VM
        Data caller, called, sReason;
        
        Sptr<SipMsg> sMsg = myInvokee->getRequest();
	cpLog(LOG_DEBUG, "******* INVITE:%s", sMsg->encode().c_str());
        caller = sMsg->getFrom().getUser();
        if(sMsg->getNumDiversion())
        {
            SipDiversion sDiversion = sMsg->getDiversion();
            Sptr<SipUrl> sUrl;
            sUrl.dynamicCast(sDiversion.getUrl());
            if(sUrl != 0)
            {
                called = sUrl->getUserValue();
            }
            sReason = sDiversion.getReason();
        }
        else
        {
             called = sMsg->getTo().getUser();
        }
        Sptr<VmcpDevice> vDevice;
        vDevice.dynamicCast(mDevice);
        assert(vDevice != 0);
        vDevice->provideCallInfo(caller, called, sReason);
        MediaController::instance().startSession(sessionId);
#endif
    }
    else
    {
        MediaController::instance().startSession(sessionId);
    }
}

