#ifndef UA_CONFIGURATION_HXX
#define UA_CONFIGURATION_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UaConfigurationVersion =
    "$Id: UaConfiguration.hxx,v 1.5 2002/11/15 20:26:11 veer Exp $";

#include <string>
#include <list>

namespace Vocal
{

namespace UA
{


typedef enum
{
    UserNameTag = 0,
    DisplayNameTag,
    PasswordTag,
    LocalSipPortTag,
    MinRtpPortTag,
    MaxRtpPortTag,
    SipTransportTag,
    ProxyServerTag,
    RegisterOnTag,
    RegisterFromTag,
    RegisterToTag,
    RegisterExpiresTag,
    UaXferModeTag,
    DeviceNameTag,
    CallWaitingOnTag,
    SubscribeOnTag,
    SubscribeExpireTag,
    SubscribeToTag,
    LogFileNameTag,
    NATAddressIPTag,
    NetworkRtpRateTag,
    LogLevelTag,
    VideoTag,
    VmServerTag,
    UaConfigurationTagMax
} UaConfigurationTags;

/**
 *  SIP User Agent configuration
 */
class UaConfiguration
{
    public:
        ///
        static UaConfiguration& instance();
        ///
        static void instance( const string cfgFile );
        ///
        static void parseCfgFileCallBack( char* tag, char* type, char* value );
        ///
        void show();

        ///
        string getValue(UaConfigurationTags tag) const;
        ///
        void setValue(UaConfigurationTags tag, string value);

        ///
        ~UaConfiguration();

        ///
        void parseConfig();

        ///
        const list<string>& getVmServers() const;

    protected:

    private:
        ///
        UaConfiguration(const string cfgFile);
        ///
        string myConfigFile;
        ///
        list<string>  myVmServers;
        ///
        string myConfigData[UaConfigurationTagMax];
        ///
        static UaConfiguration* uaConfiguration;
};
 
}

}

/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */

#endif
