#ifndef UaFacade_hxx
#define UaFacade_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const UaFacade_hxx_Version = 
    "$Id: UaFacade.hxx,v 1.6.2.3 2003/03/03 20:45:30 sprajpat Exp $";


#include "global.h"
#include <map>
#include "Sptr.hxx"
#include "Fifo.h"
#include "StatusMsg.hxx"
#include "GuiEventThread.hxx"
#include "UaWorkerThread.hxx"
#include "SipThread.hxx"
#include "RegistrationManager.hxx"
#include "SipTransceiver.hxx"
#include "MediaDevice.hxx"
#include "VMutex.h"

using Vocal::SipTransceiver;
using Vocal::SipThread;
using Vocal::UA::GuiEventThread;
using Vocal::UA::RegistrationManager;

namespace Vocal
{

namespace UA
{

typedef enum
{
  CALL_MODE_UA = 0,
  CALL_MODE_ANNON,
  CALL_MODE_VMAIL
} CallControlMode;


class UaCli;

/** UaFacade is the Facade wrapper, that wraps all the components needed
    for UA.
 */
class UaFacade
{
    public:
      ///
      static UaFacade& instance();

      ///Constructor to initialze UaFacade global instance.Should be called only once
      static UaFacade& instance(const Data& applName, unsigned short  defaultSipPort, 
                             bool filteron, bool nat);


      ///
      string className() { return "UaFacade"; }

      /**
         Frees the memory associated with singelton instance.
         gets register to atexit() function at the time of creation.
      */
      static void destroy(void);

      /** Virtual destructor
       */
      virtual ~UaFacade();

      ///
      void run();

      ///
      Sptr<SipTransceiver> getSipTransceiver() { return mySipStack; };

      ///
      Sptr< Fifo < Sptr < SipProxyEvent > > > getEventFifo() 
      { 
          return myInputEventFifo; 
      };

      ///
      Sptr<MediaDevice> getMediaDevice(int id=0);

      ///
      void releaseMediaDevice(int id);

      ///
      Sptr<RegistrationManager> getRegistrationManager() const { return myRegistrationManager; };

      ///
      void postMsg(Sptr<SipMsg> sMsg);

      ///
      void postMsg(const string msg);

      ///
      void postInfo(Sptr<SipMsg> sMsg);

      ///
      void shutdown();

      ///
      CallControlMode getMode() const { return myMode; }
      
      /**Set the mode of the user agent i.e. a phone, an announcement
       * server or interface to VoiceMail server.
       */
      void setMode(CallControlMode mode) { myMode = mode; };

    private:
        ///
        void setUpGuiEventThread();

        /** Create the Instance of UaFacade, to wrap up the application wide
         *  sipStack .
         */
        UaFacade( const Data& applName, unsigned short  defaultSipPort, bool filteron, bool nat);

        ///
        static UaFacade* myInstance;
        ///
        Sptr<SipThread> mySipThread;
        ///
        Sptr<UaWorkerThread> myWorkerThread;
        ///
        Sptr<GuiEventThread> myGuiEventThread;

        ///
        Sptr<SipTransceiver> mySipStack;

        ///
        Sptr< Fifo < Sptr < SipProxyEvent > > > myInputEventFifo;
        ///
        Sptr<RegistrationManager> myRegistrationManager;

        ///
        Sptr<MediaDevice> myMediaDevice;

        ///
        int myReadFd;
        ///
        int myWriteFd;

        ///
        Sptr<UaCli>  myUaCli;

        /** Suppress copying
         */
        UaFacade(const UaFacade &);
        
        /** Suppress copying
         */
        const UaFacade & operator=(const UaFacade &);
        ///
        CallControlMode myMode;

        ///Mapping from device Id to MediaDevice
        typedef map< int, Sptr<MediaDevice> > MediaDeviceMap;
        ///
        MediaDeviceMap myMediaDeviceMap;
        ///
        string myReadPath;
        ///
        string myWritePath;
        ///
        VMutex myMutex;
};

}
}

#endif
