/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const CheckHardware_cxx_version =
    "$Id: CheckHardware.cxx,v 1.18 2002/03/19 21:40:14 jason Exp $";



#include "global.h"
#include "CheckHardware.hxx"
#include <linux/telephony.h>
#include <linux/ixjuser.h>
#include <typeinfo>

using namespace Vocal;

///
CheckHardware::CheckHardware()
{
}


///
CheckHardware::~CheckHardware()
{}


///

const char* CheckHardware::GetType()
{
#if defined(WIN32)
    return NULL_HARDWARE;
#else
    const char *deviceName = UaConfiguration::instance()->getDeviceName()
                             .c_str();
    if ( deviceName == "" )
        return "NULL";

    // initialize ixj
    int ixj = 0;

    // open and close it to reset
    ixj = open( deviceName, O_RDWR );
    close( ixj );
    ixj = open( deviceName, O_RDWR );

    if( ixj <= 0 )
    {
        cpLog( LOG_ERR, "Open failed, device: %s Error: %s", 
	       deviceName, 
	       strerror(errno) );
        cpLog( LOG_ERR, "Defaulting to NULL_HARDWARE" );
        return "NULL";
    }
    QnHardware qnDevice = QN_NULL;

    qnDevice = static_cast < QnHardware > ( ioctl( ixj, IXJCTL_CARDTYPE ) );
    close( ixj );

    // what kind of quicknet card do we have?
    switch ( qnDevice )
    {
        case QN_PHONEJACK:
        {
            cpLog( LOG_DEBUG, "device is type PhoneJACK" );
	    return "QUICKNET";
        }
        break;

        case QN_LINEJACK:
        {
            cpLog( LOG_DEBUG, "device is type LineJACK" );
	    return "QUICKNET";
        }
        break;

        case QN_PHONEJACK_LITE:
        {
            cpLog( LOG_DEBUG, "device is type PhoneJACK-Lite" );
	    return "QUICKNET";
        }
        break;

        case QN_PHONEJACK_PCI:
        {
            cpLog( LOG_DEBUG, "device is type PhoneJACK-PCI");
	    return "QUICKNET";
        }
        break;

        /// the quicknet phonecard device must be treated differently from
        /// other quicknet devices, because it does not interface with an
        /// analog phone.
        case QN_PHONECARD:
        {
            cpLog( LOG_DEBUG, "device is type PhoneCARD (pcmcia)" );
	    return "PHONECARD";
        }
        break;

        default:
        {
            cpLog( LOG_ERR, "device is of unknown type: %d", qnDevice );
            cpLog( LOG_ERR, "defaulting to null hardware" );
	    return "NULL";
        }
        break;
    } // end switch()

    return "NULL";
#endif
}
