/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const Contact_cxx_Version =
    "$Id: Contact.cxx,v 1.17 2002/03/19 21:40:15 jason Exp $";


#include "global.h"
#include "SipUrl.hxx"
#include "Contact.hxx"

using namespace Vocal;

///
Contact::Contact( const Contact& c )
        : inviteMsg( c.inviteMsg ), statusMsg( c.statusMsg )
{
    status = c.status;
    cSeqNum = c.cSeqNum;
    lastInfoCSeq = c.lastInfoCSeq;
    remoteRingback = false;
}


///
Contact::Contact( const InviteMsg& aMsg )
        : inviteMsg( aMsg )
{
    status = 0;
    lastInfoCSeq = 0;
    cSeqNum = inviteMsg.getCSeq().getCSeqData().convertInt();
    remoteRingback = false;
}


///
void
Contact::update( const StatusMsg& msg )
{
    //for now, we will only update the status member of the class
    //I am sure this is incomplete.  Will come back to it later on.
    status = msg.getStatusLine().getStatusCode();
    statusMsg = msg;
}


///
int
Contact::getBranch()
{
    return 1;
}


///
int
Contact::getStatus()
{
    return status;
}


///
bool
Contact::operator==( const Contact& other ) const
{
    return ( computeCallLeg() == other.computeCallLeg() );
}


///
bool
Contact::operator==( const SipMsg& msg )
{
    return ( computeCallLeg() == msg.computeCallLeg() );
}


///
bool
Contact::operator!=( const Contact& other )
{
    return !( *this == other );
}


///
void
Contact::operator = (const Contact& src)
{
    inviteMsg = src.inviteMsg;
    statusMsg = src.statusMsg;
    status = src.status;
    cSeqNum = src.cSeqNum;
    lastInfoCSeq = src.lastInfoCSeq;
    remoteRingback = src.remoteRingback;
}


///
Sptr< BaseUrl >
Contact::getLocation()
{
    return inviteMsg.getRequestLine().getUrl();
}


///
InviteMsg
Contact::getInviteMsg()
{
    return inviteMsg;
}


///
StatusMsg
Contact::getStatusMsg()
{
    return statusMsg;
}


///
SipCallLeg
Contact::computeCallLeg() const
{
    return inviteMsg.computeCallLeg();
}


///
void
Contact::setLastInfoCSeq( unsigned int cseq )
{
    lastInfoCSeq = cseq;
}


///
unsigned int
Contact::getLastInfoCSeq()
{
    return lastInfoCSeq;
}

///
void
Contact::setCSeqNum( unsigned int cseq )
{
    cSeqNum = cseq;
}

///
void
Contact::setRemoteRingback( bool flag )
{
    remoteRingback = flag;
}

///
bool
Contact::getRemoteRingback() const
{
    return remoteRingback;
}
