/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const DigitTimeout_cxx_Version =
    "$Id: DigitTimeout.cxx,v 1.9 2002/03/19 21:40:15 jason Exp $";


#include "global.h"
#include <cassert>

#include "DigitTimeout.hxx"
#include "UaDevice.hxx"
#include "UaDigitTimerEvent.hxx"

using namespace Vocal;

///
DigitTimeout::DigitTimeout( Sptr < Fifo < Sptr < SipProxyEvent > > > cpq,
                            int initTimeout,
                            int interDigitTimeout )
{
    initTimeoutVal = initTimeout;
    interDigitTimeoutVal = interDigitTimeout;
    isTimeout = false;
    callProcessingQueue = cpq;
    id = 0;

    //spawing the timeout thread
    digitTimeoutThread.spawn(digitTimeoutThreadWrapper, this);
    cpLog(LOG_DEBUG, "Spawning the interdigit timeout thread");
}

///
void
DigitTimeout::timerReset()
{
    digitTimeoutMutex.lock();
    isTimeout = false;
    digitTimeoutFifo.cancel(id);
    id = digitTimeoutFifo.addDelayMs(INTER_DIGIT_TIME_OUT, initTimeoutVal);
    digitTimeoutMutex.unlock();
}

///
void
DigitTimeout::timerStart()
{
    digitTimeoutMutex.lock();
    digitTimeoutFifo.cancel(id);
    id = digitTimeoutFifo.addDelayMs(INTER_DIGIT_TIME_OUT, interDigitTimeoutVal);
    digitTimeoutMutex.unlock();
}

///
void
DigitTimeout::timerCancel()
{
    digitTimeoutFifo.cancel(id);

    digitTimeoutMutex.lock();
    id = 0;
    isTimeout = false;
    digitTimeoutMutex.unlock();
}

///
void*
DigitTimeout::digitTimeoutThreadWrapper(void* timeout)
{
    assert(timeout);

    DigitTimeout* dTimeout = static_cast < DigitTimeout* > (timeout);
    dTimeout->digitTimeoutMain();
    return 0;
}

///
void
DigitTimeout::digitTimeoutMain()
{
    cpLog( LOG_DEBUG, "starting digitTimeout" );
    while ( true )
    {
        InterDigitTimeoutType digitTimeout = digitTimeoutFifo.getNext();

        if ( digitTimeout == INTER_DIGIT_TIME_OUT )
        {
            digitTimeoutMutex.lock();
            isTimeout = true;
            digitTimeoutMutex.unlock();

            Sptr < UaDigitTimerEvent > timerEvent = new UaDigitTimerEvent( callProcessingQueue );
            assert( timerEvent != 0 );
            timerEvent->callId = UaDevice::instance()->getCallId();
            callProcessingQueue->add( timerEvent );
        }
    }
}

