#ifndef DIGITTIMEOUT_HXX
#define DIGITTIMEOUT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const DigitTimeout_hxx_Version =
    "$Id: DigitTimeout.hxx,v 1.10 2002/08/09 01:03:29 sprajpat Exp $";

#include <VMutex.h>
#include <Sptr.hxx>
#include <SipProxyEvent.hxx>
#include <Fifo.h>
#include <VThread.hxx>

namespace Vocal
{

typedef enum {
    INTER_DIGIT_TIME_OUT = -1,
    INTER_DIGIT_OK
} InterDigitTimeoutType;

class DigitTimeout
{
    public:
        DigitTimeout( Sptr < Fifo < Sptr < SipProxyEvent > > > cpq,
                      int initTimeout = 16000,
                      int interDigitTimeout = 8000 );
        void setInitTimeout(int initTimeout = 16000)
        {
            initTimeoutVal = initTimeout;
        };
        void setInterDigitTimeout(int interDigitTimeout = 8000)
        {
            interDigitTimeoutVal = interDigitTimeout;
        };
        bool isInterDigitTimeout()
        {
            return isTimeout;
        };
        void timerReset();
        void timerStart();
        void timerCancel();

    private:
        int initTimeoutVal;
        int interDigitTimeoutVal;
        Fifo < InterDigitTimeoutType > digitTimeoutFifo;
        FifoEventId id;

        ///
        VMutex digitTimeoutMutex;
        VThread digitTimeoutThread;
        static void* digitTimeoutThreadWrapper(void* timeout);
        void digitTimeoutMain();

        bool isTimeout;
        Sptr < Fifo < Sptr < SipProxyEvent > > > callProcessingQueue;
};
 
}


#endif
