/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpAck_cxx_version =
    "$Id: OpAck.cxx,v 1.23 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"

#include "OpAck.hxx"
#include "UaDeviceEvent.hxx"
#include "UaDevice.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#ifdef HAS_RSVPLIB
#include "SuaRsvp.hxx"
#endif
#include "SdpHandler.hxx"
#include "RsvpHandler.hxx"


using namespace Vocal;


OpAck::OpAck()
{
}


OpAck::~OpAck()
{
}


const char* const
OpAck::name() const
{
    return "OpAck";
}


const Sptr < State >
OpAck::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpAck operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    if ( sipMsg->getType() != SIP_ACK )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    // Set ACK in call
    Sptr< AckMsg > ack;
    ack.dynamicCast( sipMsg );
    // We can assert here, because we checked for SIP_ACK above.
    assert( ack != 0 );
    call->setAckMsg( ack );

    // Get remote SDP from call info
    Sptr < SipSdp > remoteSdp = call->getRemoteSdp();
    if ( remoteSdp != 0 ) // && remoteSdp->isValidSdp() )
    {
        // TODO For now, do nothing regardless if ACK contains sdp.
        // TODO In the future, if ACK has sdp, we should stop the previous
        // TODO media stream and start a new one with the new sdp.
    }
    else
    {
        // Get remote SDP from message
        Sptr<SipSdp> sdp;
	sdp.dynamicCast ( sipMsg->getContentData(0) );
        if ( sdp == NULL )
        {
            cpLog( LOG_ERR, "No remote SDP in ACK" );
        }
        else // if ( sdp->isValidSdp() )
        {
            remoteSdp = new SipSdp( *sdp );
            call->setRemoteSdp( remoteSdp );
            Sptr < SipSdp > localSdp = call->getLocalSdp();
            if ( localSdp != 0 )
            {
                startAudio( localSdp, remoteSdp );
            }
            else
            {
                cpLog( LOG_ERR, "No local SDP" );
            }
        }
    }

    rsvpAckHandler(call->getLocalSdp(), call->getRemoteSdp());

    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
