/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpAckReInvite_cxx_version =
    "$Id: OpAckReInvite.cxx,v 1.23 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "StatusMsg.hxx"
#include "OpAckReInvite.hxx"
#include "UaDeviceEvent.hxx"
#include "UaDevice.hxx"
#include "UaCallInfo.hxx"
#include "UaStateMachine.hxx"
#include "SipSdp.hxx"
#include "SystemInfo.hxx"

using namespace Vocal;


using Vocal::SDP::SdpSession;

OpAckReInvite::OpAckReInvite()
{
}


OpAckReInvite::~OpAckReInvite()
{
}


const char* const
OpAckReInvite::name() const
{
    return "OpAckReInvite";
}


const Sptr < State >
OpAckReInvite::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpAckReInvite operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact = call->findContact( *msg );
    if ( contact == 0 )
    {
        //not a re-invite
        return 0;
    }

    int status = msg->getStatusLine().getStatusCode();
    bool firstCall = false;

    if ( status >= 200 )
    {
        // Needs to ack farend
        AckMsg ack( *msg );


        vector < SipRoute* > routeList;
        // add the route info
        if ( *(call->getRingInvite()) == contact->getInviteMsg() )
        {
            cpLog( LOG_DEBUG, "adding route info for the first call" );
            routeList = call->getRoute1List();
            firstCall = true;
        }
        else
        {
            cpLog( LOG_DEBUG, "adding route info for the second call" );
            routeList = call->getRoute2List();
        }

        if ( routeList.size() > 0 )
        {
            ack.setRouteList( routeList );
            //
            SipRoute siproute = ack.getRoute(0);
            ack.removeRoute(0);

            SipRequestLine reqLine = ack.getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            ack.setRequestLine( reqLine );
        }

        sipEvent->getSipStack()->sendAsync( ack );
    }

    if ( status == 200 )  // need to check previous INVITE for 'hold'
    {
        // get previous INVITE message
        Sptr < SipMsgQueue > sipReceive = sipEvent->getSipMsgQueue();

        assert( sipReceive != 0 );

        Sptr < SipMsg > prevMsg;
        Sptr < InviteMsg > inviteMsg;
        SipMsgQueue::reverse_iterator i = sipReceive->rbegin();
        i++;     // skip the status message we just received
        while ( i != sipReceive->rend() )
        {
            prevMsg = *i;
            inviteMsg.dynamicCast( prevMsg );
            if ( inviteMsg != 0 )
            {
                cpLog( LOG_DEBUG, "Found invite" );
                break;
            }
            ++i;
        }
        if ( inviteMsg == 0 )
        {
            return 0;
        }

        // extract sdp info from INVITE
        Sptr<SipSdp> localSdp;
	localSdp.dynamicCast( inviteMsg->getContentData(0) );

        // check for valid sdp
        if ( !localSdp )
        {
            cpLog( LOG_ERR, "No SDP. No action taken");
            return 0;
        }

        //check for hold
        SdpSession sdpDesc = localSdp->getSdpDescriptor();
        Sptr < UaHardwareEvent > signal =
            new UaHardwareEvent( UaDevice::getDeviceQueue() );

        if ( sdpDesc.isHold() )
        {
            // if hold, return in same state
            return 0;
        }
        else
        {
            // if not hold, save SDP info & return to StateInCall

            Sptr<SipSdp> remoteSdp;
	    remoteSdp.dynamicCast ( msg->getContentData(0) );

            // check if in first/second call before setting remote SDP
            if ( firstCall == true )
            {
                call->setRemoteSdp( new SipSdp( *remoteSdp ) );
            }
            else
            {
                call->setRemote2Sdp( new SipSdp( *remoteSdp ) );
            }

            Sptr < UaStateMachine > stateMachine;
            stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
            assert( stateMachine != 0 );
            return stateMachine->findState( "StateInCall" );
        }
    }
    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
