/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpAddDigit_cxx_version =
    "$Id: OpAddDigit.cxx,v 1.18 2002/11/09 02:13:07 derekm Exp $";


#include "global.h"
#include "OpAddDigit.hxx"
#include "UaDeviceEvent.hxx"
#include "UaDevice.hxx"


///
using namespace Vocal;


OpAddDigit::OpAddDigit()
{}


OpAddDigit::~OpAddDigit()
{}


///

const char* const
OpAddDigit::name() const
{
    return "OpAddDigit";
}


///
const Sptr < State >
OpAddDigit::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpAddDigit operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }

    char digit = ' ';
    switch ( deviceEvent->type )
    {
        case DeviceEventDtmfStar:
        {
            digit = '*';
        }
        break;
        case DeviceEventDtmfHash:
        {
            digit = '#';
        }
        break;
        case DeviceEventDtmf0:
        {
            digit = '0';
        }
        break;
        case DeviceEventDtmf1:
        {
            digit = '1';
        }
        break;
        case DeviceEventDtmf2:
        {
            digit = '2';
        }
        break;
        case DeviceEventDtmf3:
        {
            digit = '3';
        }
        break;
        case DeviceEventDtmf4:
        {
            digit = '4';
        }
        break;
        case DeviceEventDtmf5:
        {
            digit = '5';
        }
        break;
        case DeviceEventDtmf6:
        {
            digit = '6';
        }
        break;
        case DeviceEventDtmf7:
        {
            digit = '7';
        }
        break;
        case DeviceEventDtmf8:
        {
            digit = '8';
        }
        break;
        case DeviceEventDtmf9:
        {
            digit = '9';
        }
        break;
        // TODO add 'A', 'B', 'C' and 'D'
        case DeviceEventCallUrl:
	{
	    cpLog(LOG_DEBUG_STACK, "setting URL to %s", 
		  deviceEvent->text.logData());
	    UaDevice::instance()->getDigitCollector()
		->setDialedUrl(deviceEvent->text.convertString());
	}
	break;
        default:
        {
            cpLog( LOG_DEBUG, "Not a digit" );
        }
    }
    if ( digit != ' ' )
    {
        cpLog( LOG_DEBUG, "Add Digit %c", digit );
        cout << digit << flush;
        UaDevice::instance()->getDigitCollector()->addDigit( digit );
    }
    return 0;
}
