/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpBusyHere_cxx_Version =
    "$Id: OpBusyHere.cxx,v 1.20 2002/11/12 20:25:13 veer Exp $";


#include "global.h"
#include "SipEvent.hxx"
#include "InviteMsg.hxx"
#include "OpBusyHere.hxx"
#include "UaDevice.hxx"
#include "UaConfiguration.hxx"
#include "SystemInfo.hxx"
#include "UaCommandLine.hxx"  

///
using namespace Vocal;


OpBusyHere::OpBusyHere()
{}


OpBusyHere::~OpBusyHere()
{}


///

const char* const
OpBusyHere::name() const
{
    return "OpBusyHere";
}


///
const Sptr < State >
OpBusyHere::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpBusyHere operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < InviteMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < SipCallId > currentCall = UaDevice::instance()->getCallId();
    if ( currentCall != 0 )
    {
        if ( msg->getCallId() == *currentCall )
        {
            // INVITE for the same call
            return 0;
        }
    }
    if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
        cout << endl << "Busy - Cannot accept call " << msg->getFrom().encode().logData() << endl;
    }
    else
    {
        cpLog(LOG_INFO, "Busy - Cannot accept call: %s" ,msg->getFrom().encode().logData());
    }

    StatusMsg status( *msg, 486 /*busy here*/ );

    // Add myself to the contact
    Sptr< SipUrl > myUrl = new SipUrl;
    myUrl->setUserValue( UaConfiguration::instance()->getUserName(), "phone" );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( UaConfiguration::instance()->getLocalSipPort() );
    SipContact me;
    me.setUrl( myUrl );
    status.setNumContact( 0 );    // Clear
    status.setContact( me );

    sipEvent->getSipStack()->sendReply( status );

    return 0;
}
