/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpConsultXferred_cxx_Version =
    "$Id: OpConsultXferred.cxx,v 1.20 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "ByeMsg.hxx"
#include "StatusMsg.hxx"
#include "TransferMsg.hxx"
#include "AckMsg.hxx"
#include "OpConsultXferred.hxx"
#include "UaHardwareEvent.hxx"
#include "UaDevice.hxx"
#include "UaStateMachine.hxx"
#include "UaCallInfo.hxx"


using namespace Vocal;


OpConsultXferred::OpConsultXferred()
{
}


OpConsultXferred::~OpConsultXferred()
{
}


const char* const
OpConsultXferred::name() const
{
    return "OpConsultXferred";
}


const Sptr < State >
OpConsultXferred::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpConsultXferred operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );


    //check for 200 message
    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < Contact > contact = call->getContact();
    assert( contact != 0 );

    if ( msg->getStatusLine().getStatusCode() != 200 )
    {
        return 0;
    }

    cpLog( LOG_DEBUG, "Consult Xferred" );

    int statusCode = call->getContact()->getStatus();

    if ( statusCode != 180 && statusCode != 183 )
    {
        cpLog( LOG_DEBUG, "Consultation transfer" );
        //send the 200 for transfer msg
        Sptr < TransferMsg > xferMsg = call->getXferMsg();
        assert( xferMsg != 0 );
        StatusMsg status(*xferMsg, 200/*OK*/);
        sipEvent->getSipStack()->sendReply( status );
        call->setXferMsg(0);
    }
    else
    {
        cpLog( LOG_DEBUG, "Converting from Blind Xfer to Consultation Xfer" );
        cpLog( LOG_DEBUG, "No need to respond to transferMsg" );
    }

    contact->update( *msg );

    //save the route
    if ( msg->getNumRecordRoute() )
    {
        if ( *(call->getRingInvite() ) == contact->getInviteMsg() )
        {
            call->setCallerRoute1List( msg->getrecordrouteList() );
        }
        else
        {
            call->setCallerRoute2List( msg->getrecordrouteList() );
        }

        int numContact = msg->getNumContact();
        if ( numContact )
        {
            SipContact sipContact = msg->getContact( numContact - 1 );
            Sptr < SipRoute > route = new SipRoute;
            route->setUrl( sipContact.getUrl() );
            if ( *(call->getRingInvite() ) == contact->getInviteMsg() )
                call->addRoute1( route );
            else
                call->addRoute2( route );
        }
    }


    //ACK the 200
    AckMsg ack( *msg );
    sipEvent->getSipStack()->sendAsync( ack );

    // Get remote SDP from message
    Sptr<SipSdp> sdp;
    sdp.dynamicCast ( msg->getContentData(0) );
    if ( sdp == NULL )
    {
        cpLog( LOG_ERR, "No remote SDP in 200 Status" );
    }
    else
    {
        call->setRemoteSdp( new SipSdp( *sdp ) );
    }

    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
