/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpCwRing_cxx_Version =
    "$Id: OpCwRing.cxx,v 1.11 2002/11/09 02:13:07 derekm Exp $";


#include "global.h"
#include "OpCwRing.hxx"
#include "UaHardwareEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaDevice.hxx"
#include "UaStateMachine.hxx"
#include "UaConfiguration.hxx"


///
using namespace Vocal;


OpCwRing::OpCwRing()
{}


OpCwRing::~OpCwRing()
{}


///

const char* const
OpCwRing::name() const
{
    return "OpCwRing";
}


///
const Sptr < State >
OpCwRing::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpCwRing operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }
    if ( deviceEvent->type != DeviceEventHookDown )
    {
        return 0;
    }

    cpLog( LOG_DEBUG, "Convert call waiting beep to actual ringing" );

    //for now, no support of 183 for the second call

    Sptr < SipCallId > call2Id = UaDevice::instance()->getCallWaitingId();
    if ( call2Id != 0 )
    {
        Sptr < SipCallId > callId = deviceEvent->callId;
        if ( *callId == *call2Id )
        {
            //grab the hardware and remove itself from call waiting list
            UaDevice::instance()->setCallId( call2Id );
            UaDevice::instance()->removeCallWaitingId( *call2Id );

            // stop cal waiting beep
            Sptr < UaHardwareEvent > signalCwbeep = new UaHardwareEvent( UaDevice::getDeviceQueue() );
            signalCwbeep->type = HardwareSignalType;
            signalCwbeep->signalOrRequest.signal = DeviceSignalStopCallWaitingBeep;
            UaDevice::getDeviceQueue()->add( signalCwbeep );

            // ring the phone
            Sptr < UaHardwareEvent > signalRing = new UaHardwareEvent( UaDevice::getDeviceQueue() );
            signalRing->type = HardwareSignalType;
            signalRing->signalOrRequest.signal = DeviceSignalRingStart;
            UaDevice::getDeviceQueue()->add( signalRing );

        }
        else
            cpLog( LOG_ERR, "The call on call waiting is not this call" );
    }
    else
        cpLog( LOG_ERR, "No call on cal waiting" );

    return 0;
}
