/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpEndCall_cxx_Version =
    "$Id: OpEndCall.cxx,v 1.28 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "ByeMsg.hxx"
#include "StatusMsg.hxx"
#include "OpEndCall.hxx"
#include "UaStateMachine.hxx"
#include "UaCallInfo.hxx"
#include "UaDevice.hxx"
#include "UaConfiguration.hxx"
#include "UaCallContainer.hxx"
#include "UaCommandLine.hxx"  

#ifdef HAS_RSVPLIB
#include "SuaRsvp.hxx"
#endif

#ifndef NO_POLICY
#include "PepAgent.hxx"
#endif


using namespace Vocal;


OpEndCall::OpEndCall()
{
}


OpEndCall::~OpEndCall()
{
}


const char* const
OpEndCall::name() const
{
    return "OpEndCall";
}


const Sptr < State >
OpEndCall::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpEndCall operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < ByeMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }
    cpLog( LOG_DEBUG, "End Call" );

    StatusMsg status( *msg, 200/*OK*/ );
    sipEvent->getSipStack()->sendReply( status );

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > curContact;
    Sptr < Contact > byeContact;

    curContact = call->getContact();
    assert( curContact != 0 );
    byeContact = call->findContact( *msg );

    if ( byeContact == 0 )
    {
        cpLog( LOG_DEBUG, "Bye message is not for a known contact" );
        return 0;
    }

    if ( *curContact != *byeContact )
    {
        cpLog( LOG_DEBUG, "Not current contact" );
        return 0;
    }


    // Show caller information
    if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
        cout << endl << VTime::strftime("%y/%m/%d %H:%M:%S")
             << " Call end " << msg->getFrom().encode().logData() << endl;
    }
    else
    {
        cpLog(LOG_DEBUG, "Call end: %s", msg->getFrom().encode().logData());
    }

#ifdef HAS_RSVPLIB
    // RSVP stuff ported from ol OpEndCall.cxx
    if ( UaConfiguration::instance()->getRsvpOn() )
    {
        UaDevice::instance()->getSuaRsvpSender().close();
        UaDevice::instance()->getSuaRsvpReserv().close();
    }
#endif

#ifndef NO_POLICY
    if ( UaConfiguration::instance()->getUsePolicyServer() )
    {
        cpLog(LOG_DEBUG,"Disable QoS handled by Marshal");
    }
#endif

    string nextState = "StateCallEnded";

    SipCallId callId = msg->getCallId();
    // do the following only if the call is on call waiting
    if ( UaDevice::instance()->isOnCallWaiting( callId ) )
    {
        // delete the call from call container
        Sptr < UaCallContainer > calls;
        calls.dynamicCast( event->getCallContainer() );
        assert( calls != 0 );
        calls->deleteCall( callId );

        UaDevice::instance()->removeCallWaitingId( callId );
        nextState = "StateIdle";
        if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
        {
            cout << endl << "no more calls on call waiting" << endl;
        }
    }

    Sptr < UaStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    return stateMachine->findState( nextState );
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
