/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpFarEndBusy_cxx_Version =
    "$Id: OpFarEndBusy.cxx,v 1.17 2002/11/09 02:13:07 derekm Exp $";


#include "global.h"
#include <SipEvent.hxx>

#include "OpFarEndBusy.hxx"
#include "UaStateMachine.hxx"
#include "UaCallInfo.hxx"
#include "UaDevice.hxx"

///
using namespace Vocal;


OpFarEndBusy::OpFarEndBusy()
{}


OpFarEndBusy::~OpFarEndBusy()
{}


///

const char* const
OpFarEndBusy::name() const
{
    return "OpFarEndBusy";
}


///
const Sptr < State >
OpFarEndBusy::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpFarEndBusy operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    int status;
    int best = 1000;
    bool haveBusy = false;

    Sptr < Contact > contact;
    Sptr < ContactList > contacts = call->getContactList();
    ContactList::iterator iter = contacts->begin();
    while ( iter != contacts->end() )
    {
        contact = (*iter);
        assert( contact != 0 );
        status = contact->getStatus();

        if ( status < best )
        {
            best = status;
            cpLog( LOG_DEBUG, "best == %d", best );
        }
        if ( status == 486 )
        {
            haveBusy = true;
            cpLog( LOG_DEBUG, "status == %d", status );
        }
        iter++;
    }
    if ( best >= 300 && haveBusy )
    {

        UaDevice::instance()->releaseRtpPort();

        cpLog( LOG_DEBUG, "condition: if( best >= 300 && haveBusy ) VALID" );
        Sptr < UaStateMachine > stateMachine;
        stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
        assert( stateMachine != 0 );

        cpLog( LOG_DEBUG, "returning StateBusy");
        return stateMachine->findState( "StateBusy" );
    }
    return 0;
}
