/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpFwdDigit_cxx_Version =
    "$Id: OpFwdDigit.cxx,v 1.21 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "OpFwdDigit.hxx"
#include "UaDeviceEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "InfoMsg.hxx"
#include "UaHardwareEvent.hxx"
#include "UaDevice.hxx"



using namespace Vocal;


OpFwdDigit::OpFwdDigit()
{
}


OpFwdDigit::~OpFwdDigit()
{
}


const char* const
OpFwdDigit::name() const
{
    return "OpFwdDigit";
}


const Sptr<State>
OpFwdDigit::process( const Sptr<SipProxyEvent> event )
{
    cpLog(LOG_DEBUG,"OpFwdDigit operation");

    Sptr<UaDeviceEvent> deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        cpLog(LOG_DEBUG,"Not a deviceEvent");
        return 0;
    }

    // prepare NULL signal
    Sptr<UaHardwareEvent> signal = new UaHardwareEvent( UaDevice::getDeviceQueue() );
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalNULL;

    // check if signal should be sent to DeviceQueue
    switch( deviceEvent->type )
    {
    case DeviceEventDtmf0:
        signal->signalOrRequest.signal = DeviceSignalDigit0;
        break;
    case DeviceEventDtmf1:
        signal->signalOrRequest.signal = DeviceSignalDigit1;
        break;
    case DeviceEventDtmf2:
        signal->signalOrRequest.signal = DeviceSignalDigit2;
        break;
    case DeviceEventDtmf3:
        signal->signalOrRequest.signal = DeviceSignalDigit3;
        break;
    case DeviceEventDtmf4:
        signal->signalOrRequest.signal = DeviceSignalDigit4;
        break;
    case DeviceEventDtmf5:
        signal->signalOrRequest.signal = DeviceSignalDigit5;
        break;
    case DeviceEventDtmf6:
        signal->signalOrRequest.signal = DeviceSignalDigit6;
        break;
    case DeviceEventDtmf7:
        signal->signalOrRequest.signal = DeviceSignalDigit7;
        break;
    case DeviceEventDtmf8:
        signal->signalOrRequest.signal = DeviceSignalDigit8;
        break;
    case DeviceEventDtmf9:
        signal->signalOrRequest.signal = DeviceSignalDigit9;
        break;
    case DeviceEventDtmfStar:
        signal->signalOrRequest.signal = DeviceSignalDigitStar;
        break;
    case DeviceEventDtmfHash:
        signal->signalOrRequest.signal = DeviceSignalDigitHash;
        break;
    default:
        return 0;
        break;
    }

    // send signal to DeviceQueue
    if( signal->signalOrRequest.signal != DeviceSignalNULL )
    {
        cpLog(LOG_DEBUG,"Sending DTMF signal %d to DeviceQueue",
              signal->signalOrRequest.signal - DeviceSignalDigit0);
        UaDevice::getDeviceQueue()->add( signal );
//        return 0;
    }
    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
