
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpInCallStopTrying_cxx_Version =
    "$Id: OpInCallStopTrying.cxx,v 1.15 2002/11/09 02:13:07 derekm Exp $";


#include "global.h"
#include <cassert>

#include <cpLog.h>

#include "CancelMsg.hxx"
#include "OpInCallStopTrying.hxx"
#include "UaDeviceEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaStateMachine.hxx"
#include "UaDevice.hxx"
#include "UaConfiguration.hxx"


///
using namespace Vocal;


OpInCallStopTrying::OpInCallStopTrying()
{}


///

const Sptr < State >
OpInCallStopTrying::process( Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpInCallStopTrying operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }

    if ( deviceEvent->type != DeviceEventFlash )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    //get the invite message to the second call
    Sptr < InviteMsg > inviteMsg = call->getRing2Invite();
    if ( inviteMsg == 0 )
    {
        cpLog( LOG_ERR, "No invite to cancel.  No action taken");
        return 0;
    }


    //remove this person from the contact list
    Sptr < Contact > contact = call->findContact( *inviteMsg );
    if ( contact != 0 )
    {
        call->removeContact( contact );
    }

    //cancel the invite
    cpLog( LOG_DEBUG, "CANCEL contact" );
    CancelMsg msg( *inviteMsg );
    deviceEvent->getSipStack()->sendAsync( msg );

    //set the current contact back to the first one
    inviteMsg = call->getRingInvite();
    assert( inviteMsg != 0 );
    contact = call->findContact( *inviteMsg );
    assert( contact != 0 );
    call->setContact( contact );

    return 0;
}


///
OpInCallStopTrying::~OpInCallStopTrying()
{}


///

const char* const
OpInCallStopTrying::name() const
{
    return "OpInCallStopTrying";
}
