
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpOnHook_cxx_Version =
    "$Id: OpOnHook.cxx,v 1.18 2002/11/09 02:13:07 derekm Exp $";



#include "global.h"
#include "OpOnHook.hxx"
#include "UaDeviceEvent.hxx"
#include "UaDevice.hxx"
#include "UaStateMachine.hxx"
#include "UaCallContainer.hxx"
#include "UaConfiguration.hxx"
#include "ResGwDevice.hxx"
#include "UaCommandLine.hxx"  


///
using namespace Vocal;


OpOnHook::OpOnHook()
{}


OpOnHook::~OpOnHook()
{}


///

const char* const
OpOnHook::name() const
{
    return "OpOnHook";
}


///
const Sptr < State >
OpOnHook::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpOnHook operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }
    if ( deviceEvent->type != DeviceEventHookDown )
    {
        return 0;
    }
    cpLog( LOG_DEBUG, "OnHook" );

    Sptr < UaStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    // Remove call info from call container
    Sptr < UaCallContainer > calls;
    calls.dynamicCast( event->getCallContainer() );
    assert( calls != 0 );
    calls->deleteCall( *(UaDevice::instance()->getCallId()) );
    // Clear call id in device
    UaDevice::instance()->setCallId( 0 );

    // release rtp port
    UaDevice::instance()->releaseRtpPort();

    // if there is a call on call waiting, send that call an onhook evnet
    Sptr < SipCallId > call2Id = UaDevice::instance()->getCallWaitingId();
    if ( call2Id != 0 )
    {
        if ( UaConfiguration::instance()->getCallWaitingOn() )
        {
            // send a message to the call on call waiting, notifying
            // it that the active call is completed
            Sptr < UaCallContainer > calls;
            calls.dynamicCast( event->getCallContainer() );
            assert( calls != 0 );
            Sptr < UaCallInfo > call2 = calls->findCall( *call2Id );
            if ( call2 == 0 )
            {
                cpLog( LOG_ERR, "No call info found for the call on call waiting");
                return 0;
            }
            Sptr < Fifo < Sptr < SipProxyEvent > > > eventQ = deviceEvent->getFifo();
            Sptr < UaDeviceEvent > event = new UaDeviceEvent( eventQ );
            event->type = DeviceEventHookDown;
            event->callId = call2Id;
            eventQ->add( event );
        }
    }
    if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
        cout << endl << "Ready" << endl;
    }

    return stateMachine->findState( "StateIdle" );
}
