/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpReDirect_cxx_Version =
    "$Id: OpReDirect.cxx,v 1.27 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "OpReDirect.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "SipTransceiver.hxx"
#include "SipVia.hxx"

using namespace Vocal;


OpReDirect::OpReDirect()
{
}


OpReDirect::~OpReDirect()
{
}


const char* const
OpReDirect::name() const
{
    return "OpReDirect";
}


const Sptr < State >
OpReDirect::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpReDirect operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    switch ( msg->getStatusLine().getStatusCode() )
    {
        case 300:     // Multiple Choices
        case 301:     // Moved Premanently
        case 302:     // Moved Temporary
        case 305:     // Use Proxy
        {
            cpLog( LOG_DEBUG, "Redirect" );
        }
        break;
        default:
        {
            return 0;
        }
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > origContact = call->findContact( *msg );
    assert( origContact != 0 );

    // Ack the 3xx
    AckMsg ack( *msg );

    Sptr< BaseUrl > baseUrl =
	origContact->getInviteMsg().getRequestLine().getUrl();
    assert( baseUrl != 0 );
    if( baseUrl->getType() == TEL_URL )
    {
	cpLog( LOG_ERR, "TEL_URL currently not supported\n" );
	assert( 0 );
    }
    // Assume we have a SIP_URL
    Sptr< SipUrl > reqUrl;
    reqUrl.dynamicCast( baseUrl );
    assert( reqUrl != 0 );

    SipRequestLine reqLine = ack.getRequestLine();
    reqLine.setUrl( reqUrl );
    ack.setRequestLine( reqLine );
    sipEvent->getSipStack()->sendAsync( ack );

    for ( int i = 0; i < msg->getNumContact(); i++ )
    {
        SipContact sipContact = msg->getContact( i );

	baseUrl = sipContact.getUrl();
	assert( baseUrl != 0 );
	if( baseUrl->getType() == TEL_URL )
	{
	    cpLog( LOG_ERR, "TEL_URL currently not supported\n" );
	    assert( 0 );
	}
	// Assume we have a SIP_URL
	Sptr< SipUrl > newUrl;
	newUrl.dynamicCast( baseUrl );
        Data tprt = newUrl->getTransportParam();

	assert( newUrl != 0 );

        if (newUrl->getUserParam() == "phone")
        {
            string proxyServer = UaConfiguration::instance()->getProxyServer();
            string::size_type colonPos = proxyServer.find( ":" );
            newUrl->setHost(proxyServer.substr( 0, colonPos ));
            cpLog( LOG_DEBUG, "newUrl->getHost() = %s",
		   newUrl->getHost().logData() );
            if ( colonPos < string::npos )
            {
                newUrl->setPort(
                    proxyServer.substr( proxyServer.rfind( ":" ) + 1 ));
            }
            else
            {
                newUrl->setPort("5060");
            }
            cpLog( LOG_DEBUG, "newUrl->getPort() = %s",
		   newUrl->getPort().logData() );
        }

        cout << "Redirected to " << newUrl->encode().logData() << endl;

        // create the new invite
        InviteMsg inviteMsg( origContact->getInviteMsg(), newUrl );

        //Set the transport in VIA same as the transport in the request line
        SipVia via = inviteMsg.getVia(0); 

        if(tprt == "tcp")
        {
            via.setTransport("TCP");
        }
        else
        {
            via.setTransport("UDP");
        }
        inviteMsg.removeVia(0);
        inviteMsg.setVia(via, 0);

        //check it is not a loop
        bool isLoop = false;
        //TODO Fix this
#if 0
	// Right now, we don't handle multiple contacts in ReDirect.
	// We just take the first one.
        Sptr < ContactList > contacts = call->getContactList();
        ContactList::iterator iter = contacts->begin();
        while ( iter != contacts->end() )
        {
            if ( *contact == *(*iter) )
            {
                // found a loop
                isLoop = true;
                cpLog( LOG_WARNING, "Redirect formed a loop" );
            }
            iter++;
        }
#endif
        if ( !isLoop )
        {
            // Use the same Call Id in the new INVITE
            inviteMsg.setCallId( sipMsg->getCallId() );
            // Increment CSeq
            SipCSeq newCSeq = inviteMsg.getCSeq();
            int cseq = sipMsg->getCSeq().getCSeqData().convertInt();
	    /**
	     *There is some question as to whether we should give a
	     * unique callId.  For now, we'll just increase the CSeq.
	     */
            newCSeq.setCSeq( ++cseq );
            inviteMsg.setCSeq( newCSeq );

            if( sipMsg->getNumDiversion() )
            {
                cpLog( LOG_DEBUG, "Copy diversion list" );
                inviteMsg.copyDiversionList( *sipMsg );

            }
            sipEvent->getSipStack()->sendAsync( inviteMsg );

            // Create the new contact
            Sptr < Contact > contact = new Contact( inviteMsg );

            // Add this to the contact list
            call->addContact( contact );
            // Set this to the current contact
            call->setContact( contact );

            // Update the Ring INVITE message
            call->setRingInvite( new InviteMsg( inviteMsg ) );
        }
    }
    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
