/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpReInvite_cxx_Version =
    "$Id: OpReInvite.cxx,v 1.31 2002/11/12 20:25:13 veer Exp $";

#include "global.h"
#include "cpLog.h"
#include "InviteMsg.hxx"
#include "SipEvent.hxx"
#include "StatusMsg.hxx"
#include "OpReInvite.hxx"
#include "UaDevice.hxx"
#include "UaHardwareEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "SdpHandler.hxx"
#include "SystemInfo.hxx"
#include "SipSdp.hxx"

using namespace Vocal;


using Vocal::SDP::SdpSession;


OpReInvite::OpReInvite()
{
}


OpReInvite::~OpReInvite()
{
}


const char* const
OpReInvite::name() const
{
    return "OpReInvite";
}


const Sptr < State >
OpReInvite::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpReInvite operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < InviteMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact = call->findContact( *msg );
    if ( contact == 0 )
    {
        call->setRingInvite( new InviteMsg( *msg ) );
        call->setContactMsg(msg);
    }

    //save route info
    call->setCalleeRoute1List( msg->getrecordrouteList() );
    int numContact = msg->getNumContact();
    if ( numContact )
    {
        SipContact contact = msg->getContact( numContact - 1 );
        Sptr < SipRoute > route = new SipRoute;
        route->setUrl( contact.getUrl() );
        call->addRoute1( route );
    }

// TMP JACK
        cpLog(LOG_DEBUG,  "%s", (msg->encode().logData()) );
    //re-invite
    //return a 200 ok status message
    StatusMsg status(*msg, 200 /*OK*/ );

    cpLog(LOG_DEBUG,  "%s", (status.encode().logData()) );
    // Add myself to the contact
    Sptr< SipUrl > myUrl = new SipUrl;
    myUrl->setUserValue( UaConfiguration::instance()->getUserName(), "phone" );
    myUrl->setHost( Data( theSystem.gethostAddress() ) );
    myUrl->setPort( UaConfiguration::instance()->getLocalSipPort() );
    SipContact me;
    me.setUrl( myUrl );
    status.setNumContact( 0 );    // Clear
    status.setContact( me );

    //check SDP
    Sptr<SipSdp> remoteSdp;
    remoteSdp.dynamicCast ( sipMsg->getContentData(0) );

    if ( !remoteSdp )
    {
        cpLog( LOG_ERR, "No SDP. No action taken");
        return 0;
    }

    //check for hold
    SdpSession sdpDesc = remoteSdp->getSdpDescriptor();
    Sptr < UaHardwareEvent > signal = new UaHardwareEvent( UaDevice::getDeviceQueue() );

    if ( sdpDesc.isHold() )
    {
        cerr << "going on hold" << endl;
    }
    else
    {
        cerr << "going off hold" << endl;

        // if not hold, save the new remote SDP
        call->setRemoteSdp( new SipSdp( *remoteSdp ) );

        // Set local SDP to send to far end
        SipSdp localSdp;

        Data host = theSystem.gethostAddress();
        if(UaConfiguration::instance()->getNATAddress() != "")
        {
            host = UaConfiguration::instance()->getNATAddress();
        }
	setStandardSdp(localSdp, 
                       host,
		       UaDevice::instance()->getRtpPort());
        status.setContentData( &localSdp, 0 );
        call->setLocalSdp( new SipSdp(localSdp) );
    }

    sipEvent->getSipStack()->sendReply( status );
    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
