/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpReInviteAuthenticated_cxx_Version =
    "$Id: OpReInviteAuthenticated.cxx,v 1.9 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "AckMsg.hxx"
#include "OpReInviteAuthenticated.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "Authenticate.hxx"
#include "SipTransceiver.hxx"


using namespace Vocal;


OpReInviteAuthenticated::OpReInviteAuthenticated()
{
}


OpReInviteAuthenticated::~OpReInviteAuthenticated()
{
}


const char* const
OpReInviteAuthenticated::name() const
{
    return "OpReInviteAuthenticated";
}


const Sptr < State >
OpReInviteAuthenticated::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpReInviteAuthenticated operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    switch ( msg->getStatusLine().getStatusCode() )
    {
        case 407:     // Proxy Authentication Required
        {
            cpLog( LOG_DEBUG, "Authentication Required" );
        }
        break;
        default:
        {
            return 0;
        }
    }

    int cseq = sipMsg->getCSeq().getCSeqData().convertInt();

#if 0
    if( cseq > 1 )
    {
	// this is not the first call sequence so we must have messed
	// up.  ack and hang up.


	return 0;
    }
#endif

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > origContact = call->findContact( *msg );
    assert( origContact != 0 );

    AckMsg ack( *msg );

    Sptr< BaseUrl > baseUrl =
	origContact->getInviteMsg().getRequestLine().getUrl();
    assert( baseUrl != 0 );
    if( baseUrl->getType() == TEL_URL )
    {
	cpLog( LOG_ERR, "TEL_URL currently not supported\n" );
	assert( 0 );
    }
    // Assume we have a SIP_URL
    Sptr< SipUrl > reqUrl;
    reqUrl.dynamicCast( baseUrl );
    assert( reqUrl != 0 );

    SipRequestLine reqLine = ack.getRequestLine();
    reqLine.setUrl( reqUrl );
    ack.setRequestLine( reqLine );
    sipEvent->getSipStack()->sendAsync( ack );

    // Check for Authentication loop
    if( origContact->getStatus() == 407 )
    {
	cpLog( LOG_ERR, "Authentication Loop. Wrong password?" );
        return 0;
    }

    Sptr<InviteMsg> inviteMsg = new InviteMsg( origContact->getInviteMsg() );

    // Use the same Call Id in the new INVITE
    inviteMsg->setCallId( sipMsg->getCallId() );
    // Increment CSeq
    SipCSeq newCSeq = inviteMsg->getCSeq();

    /**
     *There is some question as to whether we should give a
     * unique callId.  For now, we'll just increase the CSeq.
     */
    newCSeq.setCSeq( ++cseq );
    inviteMsg->setCSeq( newCSeq );

    // do the authentication thing
    
    if(authenticateMessage( *msg, 
                            *inviteMsg, 
                            UaConfiguration::instance()->getUserName(),
                            UaConfiguration::instance()->getPassword() ))
    {
        // went OK
        sipEvent->getSipStack()->sendAsync( *inviteMsg );
    }
    else
    {
        // xxx there was an authentication problem, so we need to abort out
    }

    call->setContactMsg(inviteMsg);


    // Update the Ring INVITE message
    call->setRingInvite( new InviteMsg( *inviteMsg ) );
    return 0;
}


/* Local Variables: */
/* c-file-style:"stroustrup" */
/* c-basic-offset:4 */
/* c-file-offsets:((inclass . ++)) */
/* indent-tabs-mode:nil */
/* End: */
