
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpRingCancel_cxx_Version =
    "$Id: OpRingCancel.cxx,v 1.20 2002/11/09 02:13:07 derekm Exp $";



#include "global.h"
#include <SipEvent.hxx>
#include <CancelMsg.hxx>
#include <ByeMsg.hxx>

#include "OpRingCancel.hxx"
#include "UaStateMachine.hxx"
#include "UaDevice.hxx"
#include "UaCallContainer.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "UaHardwareEvent.hxx"

///
using namespace Vocal;


OpRingCancel::OpRingCancel()
{}


OpRingCancel::~OpRingCancel()
{}


///

const char* const
OpRingCancel::name() const
{
    return "OpRingCancel";
}


///
const Sptr < State >
OpRingCancel::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpRingCancel operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    // either a cancel or a bye message will cancel the ringing
    // and terminate the call
    Sptr < StatusMsg > status;

    Sptr < CancelMsg > cancelMsg;
    cancelMsg.dynamicCast( sipMsg );

    Sptr < ByeMsg > byeMsg;
    byeMsg.dynamicCast( sipMsg );

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact;

    if ( cancelMsg != 0 )
    {
        status = new StatusMsg(*cancelMsg, 200/*OK*/);
        contact = call->findContact( *cancelMsg );
    }
    else if ( byeMsg != 0 )
    {
        status = new StatusMsg(*byeMsg, 200/*OK*/);
        contact = call->findContact( *byeMsg );
    }
    else
    {
        return 0;
    }

    sipEvent->getSipStack()->sendReply( *status );

    Sptr < InviteMsg > msg = call->getRingInvite();
    assert( msg != 0 );
    Sptr < StatusMsg > inviteStatus = new StatusMsg(*msg, 487/*Request Terminated*/);
    sipEvent->getSipStack()->sendReply( *inviteStatus );

    Sptr < Contact > curContact = call->getContact();
    assert( curContact != 0 );
    if ( contact == 0 )
    {
        cpLog( LOG_DEBUG, "No known contact for this messagemake" );
        return 0;
    }

    if ( *curContact != *contact )
    {
        // CANCEL not for current contact
        cpLog( LOG_DEBUG, "CANCEL not for current contact" );
        return 0;
    }

    cpLog( LOG_DEBUG, "Cancel Ringing" );

    // Remove call info from call container
    Sptr < UaCallContainer > calls;
    calls.dynamicCast( event->getCallContainer() );
    assert( calls != 0 );

    //calls->deleteCall( *(UaDevice::instance()->getCallId()) );
    SipCallId callId = call->getContact()->getInviteMsg().getCallId();
    calls->deleteCall( callId );

    //Stop sending ringback tone if necessary
    if (UaConfiguration::instance()->getProvideRingback() &&
            call->getContact()->getStatus() == 183 )
    {
        cpLog( LOG_DEBUG, "Stop sending ringback tone");
        Sptr < UaHardwareEvent > signal;
        signal->type = HardwareAudioType;
        signal->signalOrRequest.request.type = AudioStop;
        UaDevice::getDeviceQueue()->add( signal );
    }
    else
    {
        // release the rtp port if one was reserve
        UaDevice::instance()->releaseRtpPort();
    }

    cout << "Ready" << endl;

    Sptr < UaStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );


    return stateMachine->findState( "StateIdle" );
}
