/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpRingingInvite_cxx_Version =
    "$Id: OpRingingInvite.cxx,v 1.19 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include "SipEvent.hxx"
#include "InviteMsg.hxx"
#include "OpRingingInvite.hxx"
#include "UaCallInfo.hxx"
#include "SipTransceiver.hxx"
#include "UaCommandLine.hxx"  


using namespace Vocal;


OpRingingInvite::OpRingingInvite()
{
}


OpRingingInvite::~OpRingingInvite()
{
}


const char* const
OpRingingInvite::name() const
{
    return "OpRingingInvite";
}


const Sptr < State >
OpRingingInvite::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpRingingInvite operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < InviteMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact = call->findContact( *msg );
    if ( contact == 0 )
    {
        call->setRingInvite( new InviteMsg( *msg ) );
	call->setContactMsg(msg);
    }

    //save route info
    call->setCalleeRoute1List( msg->getrecordrouteList() );
    int numContact = msg->getNumContact();
    if ( numContact )
    {
        SipContact contact = msg->getContact( numContact - 1 );
        Sptr < SipRoute > route = new SipRoute;
        route->setUrl( contact.getUrl() );
        call->addRoute1( route );
    }

    //re-invite
    //return a 180 Ringing message
    StatusMsg status( *msg, 180 /* Ringing */ );
    sipEvent->getSipStack()->sendReply( status );

    //TODO Call log
    // Show caller information
    if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
    cout << endl << VTime::strftime("%y/%m/%d %H:%M:%S")
    << " Call " << msg->getFrom().encode().logData() << endl;
    }
    else
    {
        cpLog(LOG_DEBUG, "Call:%s", msg->getFrom().encode().logData());
    }

    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
