/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpStartAudioDuplex_cxx_Version =
    "$Id: OpStartAudioDuplex.cxx,v 1.26 2002/11/09 02:13:07 derekm Exp $";

#include "global.h"
#include <cassert>

#include "cpLog.h"
#include "SipSdp.hxx"
#include "OpStartAudioDuplex.hxx"
#include "UaDevice.hxx"
#include "UaHardwareEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "SdpHandler.hxx"


using namespace Vocal;


OpStartAudioDuplex::OpStartAudioDuplex()
{
}


OpStartAudioDuplex::~OpStartAudioDuplex()
{
}


const char* const
OpStartAudioDuplex::name() const
{
    return "OpStartAudioDuplex";
}


const Sptr < State >
OpStartAudioDuplex::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "StartAudioDuplex operation" );

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < SipSdp > remoteSdp;
    Sptr < SipSdp > localSdp;

    //get the current invite msg
    Sptr < InviteMsg > inviteMsg = call->getRingInvite();
    InviteMsg invMsg = call->getContact()->getInviteMsg();
    if ( *inviteMsg == invMsg )
    {
        remoteSdp = call->getRemoteSdp();
        localSdp = call->getLocalSdp();
    }
    else
    {
        remoteSdp = call->getRemote2Sdp();
        localSdp = call->getLocal2Sdp();
    }


    // if remote sdp is not avialable, don't start audio
    if ( remoteSdp == 0 )
    {
        cpLog( LOG_DEBUG, "No Remote Sdp.  Audio not started");
        return 0;
    }

    assert( localSdp != 0 );

    Sptr < UaHardwareEvent > signal 
	= new UaHardwareEvent( UaDevice::getDeviceQueue() );
    signal->type = HardwareAudioType;

    struct HardwareAudioRequest* request = &(signal->signalOrRequest.request);

    request->type = AudioStart;

    // Set remote host and port
    LocalScopeAllocator lo;
    strcpy( request->remoteHost,
            remoteSdp->getConnAddress().getData(lo) );
    request->remotePort = remoteSdp->getRtpPort();

    // Set local host and port
    request->localPort = localSdp->getRtpPort();
    strcpy( request->localHost,
            localSdp->getConnAddress().getData(lo) );

    //TODO Why echo cancellation always true?
    request->echoCancellation = true;


    // Check local, not remote ???
    // How is this related to Network_RTP_Rate in configuration file???
    //    cpLog( LOG_DEBUG, "Local SDP:\n%s", localSdp->encode().logData() );


    // determine local packet size

    request->rtpPacketSize = getRtpPacketSize(*localSdp);

    if(request->rtpPacketSize <= 0)
    {
        cpLog( LOG_DEBUG, 
	       "could not find local SDP RTP packet size, setting to default" 
	    );
	// Set rtpPacketSize to default in ua.cfg file
	request->rtpPacketSize 
	    = UaConfiguration::instance()->getNetworkRtpRate();
    }

    cpLog( LOG_DEBUG, "Start audio( duplex ):" );
    cpLog( LOG_DEBUG, "localHost     = %s", request->localHost);
    cpLog( LOG_DEBUG, "localPort     = %d", request->localPort);
    cpLog( LOG_DEBUG, "remoteHost    = %s", request->remoteHost);
    cpLog( LOG_DEBUG, "remotePort    = %d", request->remotePort);
    cpLog( LOG_DEBUG, "rtpPacketSize = %d", request->rtpPacketSize);

    //TODO: this will be removed once 183 is integrated.
    request->sendRingback = false;

    UaDevice::getDeviceQueue()->add( signal );

    return 0;
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
