
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpStopRinging_cxx_Version =
    "$Id: OpStopRinging.cxx,v 1.15 2002/11/09 02:13:08 derekm Exp $";


#include "global.h"
#include <cassert>
#include <cpLog.h>
#include <SipEvent.hxx>
#include <CancelMsg.hxx>
#include "OpStopRinging.hxx"
#include "UaHardwareEvent.hxx"
#include "UaDevice.hxx"
#include "UaCallInfo.hxx"

///
using namespace Vocal;


OpStopRinging::OpStopRinging()
{}


///
const Sptr < State >
OpStopRinging::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "Stop Ringing" );

    // figure out if the call is going to be cancelled or become active
    bool callCancel = false;

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent != 0 )
    {
        Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
        assert( sipMsg != 0 );
        Sptr < CancelMsg > msg;
        msg.dynamicCast( sipMsg );
        if ( msg != 0 )
        {
            callCancel = true;
        }
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < UaHardwareEvent > signal =
	new UaHardwareEvent( UaDevice::getDeviceQueue() );
    signal->type = HardwareSignalType;

    SipCallId callId = call->getContact()->getInviteMsg().getCallId();

    if ( UaDevice::instance()->getCallWaitingId() == 0 )
    {
        signal->signalOrRequest.signal = DeviceSignalRingStop;
        if ( callCancel )
            UaDevice::instance()->setCallId( 0 );
    }
    else
    {
        signal->signalOrRequest.signal = DeviceSignalStopCallWaitingBeep;
        if ( callCancel )
            UaDevice::instance()->removeCallWaitingId( callId );
    }

    UaDevice::getDeviceQueue()->add( signal );

    return 0;
}


///
OpStopRinging::~OpStopRinging()
{}


///
const char* const
OpStopRinging::name() const
{
    return "OpStopRinging";
}
