/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const OpTerminateCall_cxx_Version =
    "$Id: OpTerminateCall.cxx,v 1.34 2002/11/09 02:13:08 derekm Exp $";

#include "global.h"

#include "ByeMsg.hxx"
#include "AckMsg.hxx"
#include "OpTerminateCall.hxx"
#include "UaDeviceEvent.hxx"
#include "UaDevice.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "UaStateMachine.hxx"
#include "UaCallContainer.hxx"
#include "UaConfiguration.hxx"
#include "ResGwDevice.hxx"
#include "SystemInfo.hxx"
#include "UaCommandLine.hxx"  
#include "SipVia.hxx"

#ifdef HAS_RSVPLIB
#include "SuaRsvp.hxx"
#endif

#ifndef NO_POLICY
#include "PepAgent.hxx"
#endif


using namespace Vocal;


OpTerminateCall::OpTerminateCall()
{
}


OpTerminateCall::~OpTerminateCall()
{
}


const char* const
OpTerminateCall::name() const
{
    return "OpTerminateCall";
}


const Sptr < State >
OpTerminateCall::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpTerminateCall operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }

    if ( deviceEvent->type != DeviceEventHookDown )
    {
        return 0;
    }

    cpLog( LOG_DEBUG, "Terminate Call" );

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact = call->getContact();

    if ( contact != 0 )
    {
        SipVia sipVia;
        sipVia.setprotoVersion( "2.0" );
        sipVia.setHost( Data( theSystem.gethostAddress() ) );
        sipVia.setPort( atoi( UaConfiguration::instance()
			      ->getLocalSipPort().c_str() ) );
        sipVia.setTransport( UaConfiguration::instance()->getSipTransport() );
       
        unsigned int cseq = contact->getCSeqNum();
        contact->setCSeqNum( ++cseq );

        Sptr < ByeMsg > bye;

        if ( contact->getStatus() != 200 || (call->getAckMsg() != 0) )
        {
	    // we use the ACK to create the BYE because the
	    // INVITE may not contain the tag

	    Sptr< AckMsg > ack = call->getAckMsg();

            if ( ack != 0 )
            {
                cpLog( LOG_DEBUG, "Sending BYE to caller" );
                bye = new ByeMsg( *ack );
                //set the Transport also 
                SipRequestLine& reqLine = bye->getMutableRequestLine();
                reqLine.setTransportParam(UaConfiguration::instance()->getSipTransport());

                /** Added proxy authorization for BYE from vmserver */   
//                bye->setProxyAuthorization( msg.getProxyAuthorization() );
            }
            else
            {
                cpLog( LOG_ERR , "No ACK msg to produce BYE" );
            }
        }
        else  // status == 200
        {
            const StatusMsg& msg = contact->getStatusMsg();
            if ( &msg != 0 )
            {
                cpLog( LOG_DEBUG, "Sending BYE to callee" );
                bye = new ByeMsg( msg );
                //set the Transport also 
                SipRequestLine& reqLine = bye->getMutableRequestLine();
                reqLine.setTransportParam(UaConfiguration::instance()->getSipTransport());
            }
            else
            {
                cpLog( LOG_ERR , "No status message for BYE" );
            }
        }

	if( bye == 0 )
	{
	    cpLog( LOG_ERR, "bye == 0, unable to send BYE msg" );
	    return 0;
	}
        bye->flushViaList();
        bye->setVia( sipVia, 0 );

        SipCSeq sipCSeq = bye->getCSeq();
        sipCSeq.setCSeq( cseq );
        bye->setCSeq( sipCSeq );

        vector < SipRoute* > routeList;
        // add the route info
	// We haven't saved route info from the ACK.  We just use
	// what was saved from the INVITE.  Is this ok?
        InviteMsg invMsg = contact->getInviteMsg();
        if ( *(call->getRingInvite()) == invMsg )
        {
            cpLog( LOG_DEBUG, "adding route info for the first call" );
            routeList = call->getRoute1List();
        }
        else
        {
            cpLog( LOG_DEBUG, "adding route info for the second call" );
            routeList = call->getRoute2List();
        }

        if ( routeList.size() > 0 )
        {
            bye->setRouteList( routeList );
            //
            SipRoute siproute = bye->getRoute(0);
            bye->removeRoute(0);

            SipRequestLine reqLine = bye->getRequestLine();
            reqLine.setUrl( siproute.getUrl() );
            reqLine.setTransportParam(UaConfiguration::instance()->getSipTransport());
            bye->setRequestLine( reqLine );
        }

        deviceEvent->getSipStack()->sendAsync( *bye );
        if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
        {
            cout << endl
                 << VTime::strftime("%y/%m/%d %H:%M:%S")
                 << " Bye " << bye->getTo().encode().logData()
                 << endl;
        }
        else
        {
            cpLog(LOG_DEBUG, "Bye :%s", bye->getTo().encode().logData());
        }

#ifdef HAS_RSVPLIB
        // RSVP stuff ported from old OpTerminateCall.cxx
        if ( UaConfiguration::instance()->getRsvpOn() )
        {
            UaDevice::instance()->getSuaRsvpReserv().close();
            UaDevice::instance()->getSuaRsvpSender().close();
        }
#endif

#ifndef NO_POLICY
        if ( UaConfiguration::instance()->getUsePolicyServer() )
        {
            cpLog(LOG_DEBUG,"Disble QoS handled in Marshal");
        }
#endif
    }

    else
    {
        cpLog( LOG_ERR , "No current contact" );
    }

    Sptr < UaStateMachine > stateMachine;
    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
    assert( stateMachine != 0 );

    // Remove call info from call container
    Sptr < UaCallContainer > calls;
    calls.dynamicCast( event->getCallContainer() );
    assert( calls != 0 );
    calls->deleteCall( *(UaDevice::instance()->getCallId()) );
    // Clear call id in device
    UaDevice::instance()->setCallId( 0 );

    // if there is a call on call waiting, send that call an onhook evnet
    Sptr < SipCallId > call2Id = UaDevice::instance()->getCallWaitingId();
    if ( call2Id != 0 )
    {
        if ( UaConfiguration::instance()->getCallWaitingOn() )
        {
            // send a message to the call on call waiting, notifying
            // it that the active call is completed
            Sptr < UaCallContainer > calls;
            calls.dynamicCast( event->getCallContainer() );
            assert( calls != 0 );
            Sptr < UaCallInfo > call2 = calls->findCall( *call2Id );
            if ( call2 == 0 )
            {
                cpLog( LOG_ERR, "No call info found for the call"
		       " on call waiting");
                return 0;
            }
            Sptr < Fifo < Sptr < SipProxyEvent > > > eventQ =
		deviceEvent->getFifo();
            Sptr < UaDeviceEvent > event = new UaDeviceEvent( eventQ );
            event->type = DeviceEventHookDown;
            event->callId = call2Id;
            eventQ->add( event );
        }
    }

    if ( !UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
        cout << "Ready" << endl;
    }

    return stateMachine->findState( "StateIdle" );
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
