
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const OpTerminateTrying_cxx_Version =
    "$Id: OpTerminateTrying.cxx,v 1.14 2002/11/09 02:13:08 derekm Exp $";


#include "global.h"
#include <cassert>

#include <cpLog.h>

#include "CancelMsg.hxx"
#include "OpTerminateTrying.hxx"
#include "UaDeviceEvent.hxx"
#include "UaCallInfo.hxx"


///
using namespace Vocal;


OpTerminateTrying::OpTerminateTrying()
{}


///

const Sptr < State >
OpTerminateTrying::process( Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "TerminateTrying operation" );

    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    if ( deviceEvent == 0 )
    {
        return 0;
    }
    if ( deviceEvent->type == DeviceEventHookDown )
    {
        Sptr < UaCallInfo > call;
        call.dynamicCast( event->getCallInfo() );
        assert( call != 0 );

        Sptr < Contact > contact;
        Sptr < ContactList > contacts = call->getContactList();
        ContactList::iterator iter = contacts->begin();
        while ( iter != contacts->end() )
        {
            contact = (*iter);
            assert( contact != 0 );
            if ( contact->getStatus() < 200 )
            {
                cpLog( LOG_DEBUG, "CANCEL contact" );
                CancelMsg msg( contact->getInviteMsg() );
                deviceEvent->getSipStack()->sendAsync( msg );
            }
            iter++;
        }
    }
    return 0;
}


///
OpTerminateTrying::~OpTerminateTrying()
{}


const char* const
OpTerminateTrying::name() const
{
    return "OpTerminateTrying";
}
