
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const OpXferTargetHoldOk_cxx_Version =
    "$Id$";


#include "global.h"
#include <SipEvent.hxx>
#include <StatusMsg.hxx>
#include <AckMsg.hxx>
#include <TransferMsg.hxx>

#include "OpXferTargetHoldOk.hxx"
#include "UaCallInfo.hxx"
#include "UaStateMachine.hxx"
#include "UaConfiguration.hxx"
#include "SipTransceiver.hxx"
#include "SipVia.hxx"

///
using namespace Vocal;


OpXferTargetHoldOk::OpXferTargetHoldOk()
{}


OpXferTargetHoldOk::~OpXferTargetHoldOk()
{}


///

const char* const
OpXferTargetHoldOk::name() const
{
    return "OpXferTargetHoldOk";
}


///
const Sptr < State >
OpXferTargetHoldOk::process( const Sptr < SipProxyEvent > event )
{
    cpLog( LOG_DEBUG, "OpXferTargetHoldOk operation" );

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    if ( sipEvent == 0 )
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert( sipMsg != 0 );

    Sptr < StatusMsg > msg;
    msg.dynamicCast( sipMsg );
    if ( msg == 0 )
    {
        return 0;
    }

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Contact > contact = call->getContact();
    assert( contact != 0 );

    SipCSeq cseq = msg->getCSeq();
    if ( (int)(contact->getCSeqNum()) == cseq.getCSeqData().convertInt() )
    {
        if ( cseq.getMethod() == "INVITE" )
        {
            // Hold transaction
            int status = msg->getStatusLine().getStatusCode();
            if ( status >= 200 )
            {
                cpLog( LOG_DEBUG, "ACK for INVITE to transfer target" );
                AckMsg ack( *msg );

                vector < SipRoute* > routeList;
                // add the route info
                if ( *(call->getRingInvite()) == contact->getInviteMsg() )
                {
                    cpLog( LOG_DEBUG, "adding route info for the first call" );
                    routeList = call->getRoute1List();
                }
                else
                {
                    cpLog( LOG_DEBUG, "adding route info for the second call" );
                    routeList = call->getRoute2List();
                }

                if ( routeList.size() > 0 )
                {
                    ack.setRouteList( routeList );
                    //
                    SipRoute siproute = ack.getRoute(0);
                    ack.removeRoute(0);

                    SipRequestLine reqLine = ack.getRequestLine();
                    reqLine.setUrl( siproute.getUrl() );
                    ack.setRequestLine( reqLine );
                }

                sipEvent->getSipStack()->sendAsync( ack );

                if ( status == 200 )
                {
                    // Send TRANSFER to transferee
                    Sptr < InviteMsg > msg = call->getRingInvite();
                    assert( msg != 0 );
                    Sptr < InviteMsg > msg2 = call->getRing2Invite();
                    assert( msg2 != 0 );

                    cpLog( LOG_DEBUG, "\n\n%s", msg->encode().logData() );

                    Sptr < Contact > contact = call->findContact( *msg );
                    assert( contact != 0 );

                    SipVia sipVia;
                    sipVia.setprotoVersion( "2.0" );
                    sipVia.setHost( Data( theSystem.gethostAddress() ) );
                    sipVia.setPort( atoi( UaConfiguration::instance()->getLocalSipPort().c_str() ) );
                    sipVia.setTransport( UaConfiguration::instance()->getSipTransport() );

		    // Pull out the url from msg2
		    Sptr< BaseUrl > baseUrl = msg2->getTo().getUrl();
		    assert( baseUrl != 0 );
		    if( baseUrl->getType() == TEL_URL )
		    {
			cpLog( LOG_ERR, "TEL_URL currently not supported\n" );
			assert( 0 );
		    }
		    // Assume we have a SIP_URL
		    Sptr< SipUrl > transferToUrl;
		    transferToUrl.dynamicCast( baseUrl );
		    assert( transferToUrl != 0 );

                    if ( contact->getStatus() == 200 )
                    {
                        cpLog( LOG_DEBUG, "Send TRANSFER to original callee" );
                        StatusMsg status = contact->getStatusMsg();
                        cpLog( LOG_DEBUG, "\n\n%s", status.encode().logData());

                        assert( &status != 0 );

                        TransferMsg transferMsg( status, transferToUrl );
                        transferMsg.flushViaList();
                        transferMsg.setVia( sipVia, 0 );
                        sipEvent->getSipStack()->sendAsync( transferMsg );
                    }
                    else
                    {
                        cpLog( LOG_DEBUG, "Send TRANSFER to original caller" );

                        TransferMsg transferMsg( *msg, transferToUrl );
                        transferMsg.flushViaList();
                        transferMsg.setVia( sipVia, 0 );
                        sipEvent->getSipStack()->sendAsync( transferMsg );
                    }
                }
                else
                {
                    cout << "Transfer Target Hold status: " << status << endl;
                    cpLog( LOG_DEBUG, "Transfer target Hold failed status %d",
                           status );
                    Sptr < UaStateMachine > stateMachine;
                    stateMachine.dynamicCast( event->getCallInfo()->getFeature() );
                    assert( stateMachine != 0 );

                    return stateMachine->findState( "StateReRinging" );
                }
            }
        }
    }
    return 0;
}
