#ifndef UaBuilder_Hxx
#define UaBuilder_Hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UaBuilderVersion =
    "$Id: UaBuilder.hxx,v 1.14 2002/03/19 21:40:17 jason Exp $";



#include <Builder.hxx>
#include "SipTransceiver.hxx"

#include "RegistrationManager.hxx"
#include "SubscribeManager.hxx"

namespace Vocal
{

/// Forward declarations
class UaDeviceEvent;
class UaDigitTimerEvent;
class UaTimerEvent;
class UaCallContainer;
class UaStateMachine;
class SipEvent;


/**
 *
 */
class UaBuilder : public Builder
{
public:
    ///
    UaBuilder();
    ///
    void process( const Sptr < SipProxyEvent > nextEvent );
    ///
    void setSipStack( const Sptr < SipTransceiver > stack );
    ///
    void startRegistration();
    ///
    void setSubscribeManager( const Sptr< SubscribeManager >
			      subscribeManager );
    ///
    ~UaBuilder();

protected:
    ///

private:
    ///
    bool processSipEvent( const Sptr < SipEvent > sipEvent );
    ///
    bool processUaDeviceEvent( const Sptr < UaDeviceEvent > uaDeviceEvent );
    ///
    bool processUaDigitEvent( const Sptr < UaDigitTimerEvent > uaDigitEvent );
    ///
    bool processUaTimerEvent( const Sptr < UaTimerEvent > uaTimerEvent );
    ///
    bool handleStatusMsg( const Sptr < SipEvent > sipEvent );
    ///
    bool handleCallWaiting( Sptr< CallInfo > callInfo );
    ///
    void sendEvent( const Sptr < SipProxyEvent > nextEvent );

    ///
    Sptr < RegistrationManager > registrationManager;
    ///
    Sptr < SipTransceiver > sipStack;
    ///
    Sptr < SubscribeManager > subManager;
    ///
    Sptr< UaCallContainer > calls;
    ///
    SipCallId callId;
    ///
    Sptr< CallInfo > callInfo;
    ///
    Sptr< UaStateMachine > stateMachine;
};
 
}

#endif
