#ifndef UACALLINFO_HXX_
#define UACALLINFO_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UaCallInfo_hxx_Version =
    "$Id: UaCallInfo.hxx,v 1.16 2002/03/19 21:40:17 jason Exp $";

#include "Sptr.hxx"
#include "CallInfo.hxx"
#include "InviteMsg.hxx"
#include "AckMsg.hxx"
#include "TransferMsg.hxx"
#include "SipRoute.hxx"

#include "Contact.hxx"
#include "SipSdp.hxx"
#include "SystemInfo.hxx"

namespace Vocal
{

class SipSdp;
    
typedef list < Sptr < Contact > > ContactList;
typedef vector < Sptr < SipRoute > > RouteList;

/** Call information for a UA
 */
class UaCallInfo : public CallInfo
{
    public:
        ///
        UaCallInfo();
        ///
        ~UaCallInfo();

        ///
        void setOriginator( const bool yorn );
        ///
        bool isOriginator();

        ///
        void setCallee( const SipUrl url );
        ///
        SipUrl getCallee();

        ///
        Sptr < Contact > getContact();
        ///
        void setContact( Sptr < Contact > contact );

        ///
        Sptr < ContactList > getContactList();
        ///
        void addContact( const Sptr < Contact > contact );


        /// set the contact from a message
        void setContactMsg( Sptr<InviteMsg> msg );

        /// set the contact from a message
        void setContactMsg( InviteMsg& msg );

        ///
        Sptr < Contact > findContact( const SipMsg& msg );
        ///
        void removeContact( const Sptr < Contact > contact );
        ///
        void clearContactList();
        ///
        void setCaller( const SipUrl url );
        ///
        SipUrl getCaller();

        ///
        void setLocalSdp( const Sptr < SipSdp > sdp );
        ///
        Sptr < SipSdp > getLocalSdp();
        ///
        void setRemoteSdp( const Sptr < SipSdp > sdp );
        ///
        Sptr < SipSdp > getRemoteSdp();

        ///
        void setLocal2Sdp( const Sptr < SipSdp > sdp );
        ///
        Sptr < SipSdp > getLocal2Sdp();
        ///
        void setRemote2Sdp( const Sptr < SipSdp > sdp );
        ///
        Sptr < SipSdp > getRemote2Sdp();

        ///
        void setRingInvite( Sptr < InviteMsg > invite );
        ///
        Sptr < InviteMsg > getRingInvite();
        ///
        void setRing2Invite( Sptr < InviteMsg > invite );
        ///
        Sptr < InviteMsg > getRing2Invite();
        ///
        void setCallerRoute1List( SipRecordRouteList recordrouteList );
        ///
        void setCalleeRoute1List( SipRecordRouteList recordrouteList );
        ///
        void addRoute1( const Sptr < SipRoute > sipRoute );
        ///
        vector < SipRoute* > getRoute1List();
        ///
        void setCallerRoute2List( SipRecordRouteList recordrouteList );
        ///
        void setCalleeRoute2List( SipRecordRouteList recordrouteList );
        ///
        void addRoute2( const Sptr < SipRoute > sipRoute );
        ///
        vector < SipRoute* > getRoute2List();

        ///
        void setXferMsg( Sptr < TransferMsg > transfer );
        ///
        Sptr < TransferMsg > getXferMsg();
        ///
        void setAckMsg( Sptr < AckMsg > ack );
        ///
        Sptr< AckMsg > getAckMsg();

    protected:

    private:
        ///
        bool originator;
        ///
        SipUrl callee;
        ///
        Sptr < Contact > currentContact;
        ///
        Sptr < ContactList > contactList;
        ///
        SipUrl caller;
        ///
        Sptr < SipSdp > localSdp;
        ///
        Sptr < SipSdp > remoteSdp;
        ///
        Sptr < SipSdp > local2Sdp;
        ///
        Sptr < SipSdp > remote2Sdp;
        ///
        Sptr < InviteMsg > ringInvite;
        ///
        Sptr < InviteMsg > ring2Invite;
        ///
        Sptr < RouteList > route1List;
        ///
        Sptr < RouteList > route2List;
        ///
        Sptr < TransferMsg > xferMsg;
        ///
        Sptr < AckMsg > ackMsg;
};
 
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */

#endif
