/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UaConfiguration_cxx_Version =
    "$Id: UaConfiguration.cxx,v 1.35 2002/10/28 22:49:42 sprajpat Exp $";

#include "global.h"
#include <cassert>
#include <cstdio>

#include "cpLog.h"
#include "parse3tuple.h"

#include "Data.hxx"
#include "UaConfiguration.hxx"


using namespace Vocal;

UaConfiguration* UaConfiguration::uaConfiguration = 0;


UaConfiguration*
UaConfiguration::instance()
{
    assert( uaConfiguration != 0 );
    return uaConfiguration;
}    // UaConfiguration::instance


typedef enum
{
    UserNameTag = 0,
    DisplayNameTag,
    PasswordTag,
    LocalSipPortTag,
    MinRtpPortTag,
    MaxRtpPortTag,
    SipTransportTag,
    ProxyServerTag,
    RegisterOnTag,
    RegisterFromTag,
    RegisterToTag,
    RegisterExpiresTag,
    RegisterActionTag,
    NetworkRtpRateTag,
    ProvideRingbackTag,
    InitialDigitTimeoutTag,
    InterDigitTimeoutTag,
    DialPatternTag,
    SpeedDialTag,
    LoadGenOnTag,
    RunModeTag,
    StartTimeTag,
    NumKickStartsTag,
    CallUrlTag,
    NumEndpointsTag,
    CallDurationTag,
    CallDelayTag,
    CallRestartTimerTag,
    AnswerRestartTimerTag,
    AnswerDelayTag,
    MonitorMsgOnTag,
    MonitorMsgIntervalTag,
    DialNumberTag,
    RtpGenOnTag,
    NumOfCallsTag,
    CJTimeTag,
    RsvpOnTag,
    ProvisioningHostTag,
    ProvisioningPortTag,
    UsePolicyServerTag,
    UaXferModeTag,
    ConfServerTag,
    DeviceNameTag,
    CallWaitingOnTag,
    CallIdOnTag,
    CallIdTag,
    UaTimeoutTag,
    SubscribeOnTag,
    SubscribeExpireTag,
    SubscribeToTag,
    VmServerTag,
    LogFileNameTag,
    NATAddressTag,
    UaConfigurationTagMax
} UaConfigurationTags;


static const char* UaConfigurationTagString [] =
    {
        "User_Name",
        "Display_Name",
        "Pass_Word",
        "Local_SIP_Port",
        "Min_RTP_Port",
        "Max_RTP_Port",
        "SIP_Transport",
        "Proxy_Server",
        "Register_On",
        "Register_From",
        "Register_To",
        "Register_Expires",
        "Register_Action",
        "Network_RTP_Rate",
        "Provide_Ringback",
        "Initial_Digit_Timeout",
        "Inter_Digit_Timeout",
        "Dial_Pattern",
        "Speed_Dial",
        "LoadGen_On",
        "RunMode",
        "StartTime",
        "NumKickStarts",
        "CallUrl",
        "NumEndpoints",
        "CallDuration",
        "CallDelay",
        "CallRestartTimer",
        "AnswerRestartTimer",
        "AnswerDelay",
        "MonitorMsgOn",
        "MonitorMsgInterval",
        "DialNumber",
        "RtpGenOn",
        "NumOfCalls",
        "CJTime",
        "Rsvp_On",
        "Provisioning_Host",
        "Provisioning_Port",
        "Use_Policy_Server",
        "Ua_Xfer_Mode",
        "Conference_Server",
        "Device_Name",
        "Call_Waiting_On",
        "CallId_On",
        "CallId",
        "Ua_Timeout",
        "Subscribe_On",
        "Subscribe_Expires",
        "Subscribe_To",
        "VmServer",
	"LogFilename",
	"NATAddress",
        "Unknown"
    };


void
UaConfiguration::parseCfgFileCallBack( char* tag, char* type, char* value )
{
    switch ( strConst2i( tag, UaConfigurationTagString, UaConfigurationTagMax ) )
    {
        case UserNameTag:
        {
            UaConfiguration::instance()->setUserName( value );
        }
        break;

        case DisplayNameTag:
        {
            UaConfiguration::instance()->setDisplayName( value );
        }
        break;

        case PasswordTag:
        {
            UaConfiguration::instance()->setPassword( value );
        }
        break;

        case LocalSipPortTag:
        {
            UaConfiguration::instance()->setLocalSipPort( value );
        }
        break;

	case MinRtpPortTag:
	{
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setMinRtpPort( i );
	}
	break;

	case MaxRtpPortTag:
	{
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setMaxRtpPort( i );
	}
	break;

        case SipTransportTag:
        {
            UaConfiguration::instance()->setSipTransport( value );
        }
        break;

        case ProxyServerTag:
        {
            UaConfiguration::instance()->setProxyServer( value );
        }
        break;

        case RegisterOnTag:
        {
            UaConfiguration::instance()->setRegisterOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case RegisterFromTag:
        {
            UaConfiguration::instance()->setRegisterFrom( value );
        }
        break;

        case RegisterToTag:
        {
            UaConfiguration::instance()->setRegisterTo( value );
        }
        break;

        case RegisterExpiresTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setRegisterExpires( i );
        }
        break;

        case RegisterActionTag:
        {
            UaConfiguration::instance()->setRegisterAction( value );
        }
        break;

        case NetworkRtpRateTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setNetworkRtpRate( i );
        }
        break;

        case ProvideRingbackTag:
        {
            int flag;
            sscanf( value, "%d", &flag );
            UaConfiguration::instance()->setProvideRingback( isEqualNoCase( string( value ), "True" ) ) ;
        }
        break;

        case InitialDigitTimeoutTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setInitialDigitTimeout( i );
        }
        break;

        case InterDigitTimeoutTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setInterDigitTimeout( i );
        }
        break;

        case DialPatternTag:
        {
            int type;
            char pattern[ VALUE_MAX_LENGTH ];
            sscanf( value, "%d %s", &type, pattern );
            UaConfiguration::instance()->addDialPattern( (DialMethodsType)type,
                    pattern );
        }
        break;

        case SpeedDialTag:
        {
            char key[ VALUE_MAX_LENGTH ];
            char dest[ VALUE_MAX_LENGTH ];
            sscanf( value, "%s %s", key, dest );
            UaConfiguration::instance()->addSpeedDial( key, dest );
        }
        break;

        case LoadGenOnTag:
        {
            UaConfiguration::instance()->setLoadGenOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case StartTimeTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setStartTime( i );
        }
        break;

        case NumKickStartsTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setNumKickStarts( i );
        }
        break;

        case RunModeTag:
        {
            UaConfiguration::instance()->setRunMode( value );
        }
        break;

        case CallUrlTag:
        {
            UaConfiguration::instance()->setCallUrl( value );
        }
        break;

        case NumEndpointsTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setNumEndpoints(i);
        }
        break;

        case CallDurationTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setCallDuration(i);

        }
        break;

        case CallDelayTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setCallDelay(i);

        }
        break;

        case CallRestartTimerTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setCallRestartTimer(i);

        }
        break;

        case AnswerRestartTimerTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setAnswerRestartTimer( i );

        }
        break;

        case AnswerDelayTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setAnswerDelay(i);

        }
        break;

        case MonitorMsgOnTag:
        {
            UaConfiguration::instance()->setMonitorMsgOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case MonitorMsgIntervalTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setMonitorMsgInterval(i);

        }
        break;

        case DialNumberTag:
        {
            UaConfiguration::instance()->setDialNumber( value );
        }
        break;

        case RtpGenOnTag:
        {
            UaConfiguration::instance()->setRtpGenOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case NumOfCallsTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setNumOfCalls( i );
        }
        break;

        case CJTimeTag:
        {
            UaConfiguration::instance()->setCJTimeOfDay( isEqualNoCase(string(value), "True") );
        }
        break;

        case RsvpOnTag:
        {
            UaConfiguration::instance()->setRsvpOn( isEqualNoCase(string( value ), "True" ) );
        }
        break;

        case ProvisioningHostTag:
        {
            UaConfiguration::instance()->setProvisioningHost( value );
        }
        break;

        case ProvisioningPortTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setProvisioningPort( i );
        }
        break;

        case UsePolicyServerTag:
        {
            UaConfiguration::instance()->setUsePolicyServer( isEqualNoCase(string( value ), "True" ) );
        }
        break;

        case UaXferModeTag:
        {
            UaXferMode xfermode = XferOff;
            if ( string( value ) == "Transfer" )
	    {
                xfermode = XferOn;
	    }
            else if ( string( value ) == "Conference" )
	    {
                xfermode = ConfOn;
	    }
            UaConfiguration::instance()->setXferMode( xfermode );
        }
        break;

        case ConfServerTag:
        {
            UaConfiguration::instance()->setConfServer( string(value) );
        }
        break;

        case DeviceNameTag:
        {
            UaConfiguration::instance()->setDeviceName( value );
        }
        break;

        case CallWaitingOnTag:
        {
            UaConfiguration::instance()->setCallWaitingOn( isEqualNoCase(string( value ), "True" ) );
        }
        break;

        case CallIdOnTag:
        {
            UaConfiguration::instance()->setCallIdOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case CallIdTag:
        {
            UaConfiguration::instance()->setCallId( value );
        }
        break;

        case UaTimeoutTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setUaTimeout(i);
        }
        break;

        case SubscribeOnTag:
        {
            UaConfiguration::instance()->setSubscribeOn( isEqualNoCase( string( value ), "True" ) );
        }
        break;

        case SubscribeToTag:
        {
            UaConfiguration::instance()->setSubscribeTo( value );
        }
        break;

        case SubscribeExpireTag:
        {
            int i;
            sscanf( value, "%d", &i );
            UaConfiguration::instance()->setSubscribeExpire( i );
        }
        break;

    case VmServerTag:
    {
	UaConfiguration::instance()->addVMServer(string(value));
    }
    break;

    case LogFileNameTag:
    {
	UaConfiguration::instance()->setLogFilename(string(value));
    }
    break;

    case NATAddressTag:
    {
	UaConfiguration::instance()->setNATAddress(string(value));
    }
    break;

        default:
        {
            cpLog( LOG_WARNING, "Unknown Tag: %s", tag );
        }
    }    /* switch */
}    // UaConfiguration::parseCfgFileCallBack


void
UaConfiguration::instance( const string cfgFile )
{
    assert( uaConfiguration == 0 );

    uaConfiguration = new UaConfiguration;
    assert( uaConfiguration != 0 );

    if(cfgFile.length())
    {
        if( !parse3tuple( cfgFile.c_str(), parseCfgFileCallBack ) )
        {
            // Stop UA
            exit( -1 );
        }
    }

    uaConfiguration->show();
}    // UaConfiguration::instance


void
UaConfiguration::show()
{
    cpLog( LOG_INFO, "SIP User Agent Configuration" );
    cpLog( LOG_INFO, "--- Device ---" );
    cpLog( LOG_INFO, "       Device Name : \"%s\"", getDeviceName().c_str() );
    cpLog( LOG_INFO, "--- User ---" );
    cpLog( LOG_INFO, "              Name : \"%s\"", getUserName().c_str() );
    cpLog( LOG_INFO, "      Display Name : \"%s\"", getDisplayName().c_str() );
    cpLog( LOG_INFO, "         Pass Word : \"%s\"", getPassword().c_str() );

    cpLog( LOG_INFO, "--- SIP ---" );
    cpLog( LOG_INFO, "              Port : \"%s\"", getLocalSipPort().c_str() );
    cpLog( LOG_INFO, "         Transport : \"%s\"", getSipTransport().c_str() );
    cpLog( LOG_INFO, "      Proxy Server : \"%s\"", getProxyServer().c_str() );

    cpLog( LOG_INFO, "--- Registration ---" );
    cpLog( LOG_INFO, "          Register : \"%s\"", (getRegisterOn() ? "True" : "False" ) );
    cpLog( LOG_INFO, "              From : \"%s\"", getRegisterFrom().c_str() );
    cpLog( LOG_INFO, "                To : \"%s\"", getRegisterTo().c_str() );
    cpLog( LOG_INFO, "           Expires : %d s", getRegisterExpires() );
    cpLog( LOG_INFO, "            Action : \"%s\"", getRegisterAction().c_str() );

    cpLog( LOG_INFO, "--- Subscribe ---" );
    cpLog( LOG_INFO, "         Subscribe : \"%s\"", (getSubscribeOn() ? "True" : "False" ) );
    cpLog( LOG_INFO, "                To : \"%s\"", getSubscribeTo().c_str() );
    cpLog( LOG_INFO, "           Expires : %d s", getSubscribeExpire() );

    cpLog( LOG_INFO, "--- Dialing ---" );
    cpLog( LOG_INFO, "      Dial Timeout : %d ms", getInitialDigitTimeout() );
    cpLog( LOG_INFO, "Interdigit Timeout : %d ms", getInterDigitTimeout() );

    cpLog( LOG_INFO, "--- RTP ---" );
    cpLog( LOG_INFO, "          RTP Rate : %d ms", getNetworkRtpRate() );

    cpLog( LOG_INFO, "--- Misc Features ---" );
    cpLog( LOG_INFO, "  Provide Ringback : \"%s\"", getProvideRingback() ? "True" : "False" );
    string xfermode = "XferOff";
    if ( getXferMode() == XferOn )
        xfermode = "Transfer";
    else if ( getXferMode() == ConfOn )
        xfermode = "Conference";
    cpLog( LOG_INFO, "         Xfer Mode : \"%s\"", xfermode.c_str() );
    cpLog( LOG_INFO, "  Conference Server: \"%s\"", getConfServer().c_str() );
    cpLog( LOG_INFO, "       Call Waiting: \"%s\"", getCallWaitingOn() ? "True" : "False" );
    cpLog( LOG_INFO, "            Timeout: %d s", getUaTimeout() );

    cpLog( LOG_INFO, "          Min RTP Port : %d", getMinRtpPort() );
    cpLog( LOG_INFO, "          Max RTP Port : %d", getMaxRtpPort() );

    cpLog( LOG_INFO, "--- Dial Patterns ---" );
    DialingPlanListType dp = getDialPlans();
    DialingPlanListType::iterator dpIter = dp.begin();
    while ( dpIter != dp.end() )
    {
        cpLog( LOG_INFO, "%18d : %s", (*dpIter)->getDialMethod(),
               (*dpIter)->getDialPattern().c_str() );
        dpIter++;
    }

    cpLog( LOG_INFO, "--- Speed Dials ---" );
    SpeedDialMapType sd = getSpeedDials();
    SpeedDialMapType::iterator sdIter = sd.begin();
    while ( sdIter != sd.end() )
    {
        cpLog( LOG_INFO, "%18s : %s", sdIter->first.c_str(),
               sdIter->second.c_str() );
        sdIter++;
    }


    cpLog( LOG_INFO, "--- Manual CallId ---" );
    cpLog( LOG_INFO, "      Manual CallId: \"%s\"", (getCallIdOn() ? "True" : "False" ) );
    cpLog( LOG_INFO, "             CallId: \"%s\"", getCallId().c_str() );

    // Display Load Gen info only if it's set.
    if (getLoadGenOn())
    {
        cpLog( LOG_INFO, "--- Load Generation is turned on ---");
        cpLog( LOG_INFO, "            RunMode: \"%s\"", getRunMode().c_str() );
        cpLog( LOG_INFO, "          StartTime: %d ms", getStartTime() );
        cpLog( LOG_INFO, "      NumKickStarts: %d", getNumKickStarts() );
        cpLog( LOG_INFO, "           Call Url: %d", getCallUrl().c_str() );
        cpLog( LOG_INFO, "*      NumEndpoints: %d", getNumEndpoints() );
        cpLog( LOG_INFO, "       CallDuration: %d ms", getCallDuration() );
        cpLog( LOG_INFO, "          CallDelay: %d ms", getCallDelay() );
        cpLog( LOG_INFO, "   CallRestartTimer: %d ms", getCallRestartTimer() );
        cpLog( LOG_INFO, " AnswerRestartTimer: %d ms", getAnswerRestartTimer() );
        cpLog( LOG_INFO, "*       AnswerDelay: %d ms", getAnswerDelay() );
        if ( getMonitorMsgOn() )
        {
            cpLog( LOG_INFO, "       MonitorMsgOn: True" );
            cpLog( LOG_INFO, " MonitorMsgInterval: %d sec", getMonitorMsgInterval() );
        }
        else
        {
            cpLog( LOG_INFO, "       MonitorMsgOn: False" );
        }
        cpLog( LOG_INFO, "    First Dialing #: \"%s\"", getDialNumber().c_str() );
        if ( getRtpGenOn() )
        {
            cpLog( LOG_INFO, "       RtpGenOn: True");
        }
        else
        {
            cpLog( LOG_INFO, "       RtpGenOn: False");
        }
        cpLog( LOG_INFO, "         NumOfCalls: %d", getNumOfCalls() );
        if ( getCJTimeOfDay() )
        {
            cpLog( LOG_INFO, "        CJTimeOfDay: True" );
        }
    }
    else
    {
        cpLog( LOG_INFO, "--- Load Generation is turned off ---");
    };

    cpLog( LOG_INFO, "\n\n");


}    // UaConfiguration::show


string
UaConfiguration::getUserName()
{
    return userName;
}    // UaConfiguration::getUserName


string
UaConfiguration::getDisplayName()
{
    return displayName;
}    // UaConfiguration::getDisplayName


string
UaConfiguration::getPassword()
{
    return password;
}    // UaConfiguration::getPassword


string
UaConfiguration::getLocalSipPort()
{
    return localSipPort;
}    // UaConfiguration::getLocalSipPort


int
UaConfiguration::getMinRtpPort()
{
    return minRtpPort;
}
    

int
UaConfiguration::getMaxRtpPort()
{
    return maxRtpPort;
}


string
UaConfiguration::getSipTransport()
{
    return sipTransport;
}    // UaConfiguration::getSipTransport


string
UaConfiguration::getProxyServer()
{
    return proxyServer;
}    // UaConfiguration::getProxyServer


bool
UaConfiguration::getRegisterOn()
{
    return registerOn;
}    // UaConfiguration::getRegisterOn


string
UaConfiguration::getRegisterFrom()
{
    return registerFrom;
}    // UaConfiguration::getRegisterFrom


string
UaConfiguration::getRegisterTo()
{
    return registerTo;
}    // UaConfiguration::getRegisterTo


int
UaConfiguration::getRegisterExpires()
{
    return registerExpires;
}    // UaConfiguration::getRegisterExpires


string
UaConfiguration::getRegisterAction()
{
    return registerAction;
}    // UaConfiguration::getRegisterAction


int
UaConfiguration::getNetworkRtpRate()
{
    return networkRtpRate;
}    // UaConfiguration::getNetworkRtpRate


bool
UaConfiguration::getProvideRingback()
{
    return provideRingback;
}    // UaConfiguration::getNetworkRtpRate


int
UaConfiguration::getInitialDigitTimeout()
{
    return initialDigitTimeout;
}    // UaConfiguration::getInitialDigitTimeout


int
UaConfiguration::getInterDigitTimeout()
{
    return interDigitTimeout;
}    // UaConfiguration::getInterDigitTimeout


DialingPlanListType&
UaConfiguration::getDialPlans()
{
    return dialPlans;
}    // UaConfiguration::getDialPlans


SpeedDialMapType&
UaConfiguration::getSpeedDials()
{
    return speedDials;
}    // UaConfiguration::getSpeedDials


UaXferMode
UaConfiguration::getXferMode() const
{
    return xferMode;
}


string
UaConfiguration::getConfServer() const
{
    return confServer;
}


void
UaConfiguration::setUserName( string name )
{
    userName = name;
}    // UaConfiguration::setUserName


void
UaConfiguration::setDisplayName( string name )
{
    displayName = name;
}    // UaConfiguration::setDisplayName


void
UaConfiguration::setPassword( string pwd )
{
    password = pwd;
}    // UaConfiguration::setPassword


void
UaConfiguration::setLocalSipPort( string port )
{
    localSipPort = port;
}    // UaConfiguration::setLocalSipPort


void
UaConfiguration::setMinRtpPort( int port )
{
    minRtpPort = port;
}    // UaConfiguration::getMinRtpPort


void
UaConfiguration::setMaxRtpPort( int port )
{
    maxRtpPort = port;
}    // UaConfiguration::getMaxRtpPort


void
UaConfiguration::setSipTransport( string transport )
{
    sipTransport = transport;
}    // UaConfiguration::setSipTransport


void
UaConfiguration::setProxyServer( string server )
{
    proxyServer = server;
}    // UaConfiguration::setProxyServer


void
UaConfiguration::setRegisterOn( bool on )
{
    registerOn = on;
}    // UaConfiguration::setRegisterOn


void
UaConfiguration::setRegisterFrom( string from )
{
    registerFrom = from;
}    // UaConfiguration::setRegisterFrom


void
UaConfiguration::setRegisterTo( string to )
{
    registerTo = to;
}    // UaConfiguration::setRegisterTo


void
UaConfiguration::setRegisterExpires( int expires )
{
    registerExpires = expires;
}    // UaConfiguration::setRegisterExpires


void
UaConfiguration::setRegisterAction( string action )
{
    registerAction = action;
}    // UaConfiguration::setRegisterAction


void
UaConfiguration::setNetworkRtpRate( int rate )
{
    networkRtpRate = rate;
}    // UaConfiguration::setNetworkRtpRate


void
UaConfiguration::setProvideRingback( bool ringback )
{
    provideRingback = ringback;
}    // UaConfiguration::setNetworkRtpRate


void
UaConfiguration::setInitialDigitTimeout( int timeout )
{
    initialDigitTimeout = timeout;
}    // UaConfiguration::setInitialDigitTimeout


void
UaConfiguration::setInterDigitTimeout( int timeout )
{
    interDigitTimeout = timeout;
}    // UaConfiguration::setInterDigitTimeout


void
UaConfiguration::addDialPattern( DialMethodsType type, string pattern )
{
    DialingPlan* plan = new DialingPlan( pattern, type );
    dialPlans.push_back( plan );
}    // UaConfiguration::addDialPattern


void
UaConfiguration::addSpeedDial( string key, string dest )
{
    speedDials[ key ] = dest;
}    // UaConfiguration::addSpeedDial

/*    Load Generation stuff  */


bool
UaConfiguration::getLoadGenOn()
{
    return loadGenOn;
}


string
UaConfiguration::getRunMode()
{
    return runMode;
}


int
UaConfiguration::getStartTime()
{
    return startTime;
}


int
UaConfiguration::getNumKickStarts()
{
    return numKickStarts;
}


string
UaConfiguration::getCallUrl()
{
    return callUrl;
}


int
UaConfiguration::getNumEndpoints()
{
    return numEndpoints;
}


int
UaConfiguration::getCallDuration()
{
    return callDuration;
}


int
UaConfiguration::getCallDelay()
{
    return callDelay;
}


int
UaConfiguration::getCallRestartTimer()
{
    return callRestartTimer;
}


int
UaConfiguration::getAnswerRestartTimer()
{
    return answerRestartTimer;
}


int
UaConfiguration::getAnswerDelay()
{
    return answerDelay;
}


bool
UaConfiguration::getMonitorMsgOn()
{
    return monitorMsgOn;
}


int
UaConfiguration::getMonitorMsgInterval()
{
    return monitorMsgInterval;
}


string
UaConfiguration::getDialNumber()
{
    return dialNumber;
}


bool
UaConfiguration::getRtpGenOn()
{
    return rtpGenOn;
}


int
UaConfiguration::getNumOfCalls()
{
    return numOfCalls;
}


bool
UaConfiguration::getCJTimeOfDay()
{
    return cjTime;
}


void
UaConfiguration::setLoadGenOn(bool on)
{
    loadGenOn = on;
}


void
UaConfiguration::setRunMode(string mode)
{
    runMode = mode;
}


void
UaConfiguration::setStartTime( int time)
{
    startTime = time;
}


void
UaConfiguration::setNumKickStarts( int num)
{
    numKickStarts = num;
}


void
UaConfiguration::setCallUrl(string callUrlString)
{
    callUrl = callUrlString;
}


void
UaConfiguration::setNumEndpoints(int endpoints)
{
    numEndpoints = endpoints;
}


void
UaConfiguration::setCallDuration(int duration)
{
    callDuration = duration;
}


void
UaConfiguration::setCallDelay(int delay)
{
    callDelay = delay;
}


void
UaConfiguration::setCallRestartTimer(int delay)
{
    callRestartTimer = delay;
}


void
UaConfiguration::setAnswerRestartTimer(int delay)
{
    answerRestartTimer = delay;
}


void
UaConfiguration::setAnswerDelay(int ansDelay)
{
    answerDelay = ansDelay;
}


void
UaConfiguration::setMonitorMsgOn(bool msgOn)
{
    monitorMsgOn = msgOn;
}


void
UaConfiguration::setMonitorMsgInterval(int debugInt)
{
    monitorMsgInterval = debugInt;
}


void
UaConfiguration::setDialNumber(string dialnum )
{
    dialNumber = dialnum;
}


void
UaConfiguration::setRtpGenOn(bool rtpOn)
{
    rtpGenOn = rtpOn;
}


void
UaConfiguration::setNumOfCalls(int calls)
{
    numOfCalls = calls;
}


void
UaConfiguration::setCJTimeOfDay(bool cjOn)
{
    cjTime = cjOn;
}


bool
UaConfiguration::getRsvpOn() const
{
    return rsvpOn;
}


string
UaConfiguration::getProvisioningHost() const
{
    return provisioningHost;
}


int
UaConfiguration::getProvisioningPort() const
{
    return provisioningPort;
}


bool
UaConfiguration::getUsePolicyServer() const
{
    return usePolicyServer;
}


string
UaConfiguration::getDeviceName() const
{
    return deviceName;
}


bool
UaConfiguration::getCallWaitingOn() const
{
    return callWaitingOn;
}


bool
UaConfiguration::getCallIdOn()
{
    return myCallIdOn;
}


string
UaConfiguration::getCallId()
{
    return myCallId;
}


int
UaConfiguration::getUaTimeout() const
{
    return uaTimeout;
}


bool
UaConfiguration::getSubscribeOn() const
{
    return subscribeOn;
}    // UaConfiguration::getSubscribeOn


string
UaConfiguration::getSubscribeTo() const
{
    return subscribeTo;
}    // UaConfiguration::getSubscribeTo


int
UaConfiguration::getSubscribeExpire() const
{
    return subscribeExpire;
}    // UaConfiguration::getSubscribeExpire


void
UaConfiguration::setRsvpOn( const bool on )
{
    rsvpOn = on;
}


void
UaConfiguration::setProvisioningHost( const string host )
{
    provisioningHost = host;
}


void
UaConfiguration::setProvisioningPort( const int port )
{
    provisioningPort = port;
}


void
UaConfiguration::setUsePolicyServer( const bool use )
{
    usePolicyServer = use;
}


void
UaConfiguration::setXferMode( UaXferMode xfermode )
{
    xferMode = xfermode;
}


void
UaConfiguration::setConfServer( string confserver )
{
    confServer = confserver;
}


void
UaConfiguration::setCallId( const string callid )
{
    if (callid.length() <= 0 )
    {
        myCallId = "";
    }
    else
    {
        myCallId = callid;
    }
}


void
UaConfiguration::setCallIdOn( const bool callIdOn )
{
    myCallIdOn = callIdOn;
}


void
UaConfiguration::setDeviceName( const string devicename )
{
    deviceName = devicename;
}


void
UaConfiguration::setCallWaitingOn( const bool callwaiting )
{
    callWaitingOn = callwaiting;
}


void
UaConfiguration::setUaTimeout(const int timeout)
{
    uaTimeout = timeout;
}


void
UaConfiguration::setSubscribeOn( const bool subOn )
{
    subscribeOn = subOn ;
}    // UaConfiguration::setSubscribeOn


void
UaConfiguration::setSubscribeTo( const string to )
{
    subscribeTo = to;
}    // UaConfiguration::setSubscribeTo


void
UaConfiguration::setSubscribeExpire( const int expire )
{
    subscribeExpire = expire;
}    // UaConfiguration::setSubscribeExpire


void
UaConfiguration::addVMServer(const string server)
{
    myVmServers.push_back(server);
}


vector<string> &
UaConfiguration::getVMServers()
{
    return myVmServers;
}


void
UaConfiguration::setLogFilename(const string& filename)
{
    myLogFilename = filename;
}


string
UaConfiguration::getLogFilename() const
{
    return myLogFilename;
}


void
UaConfiguration::setNATAddress(const string& address)
{
    myNATAddress = address;
}


string
UaConfiguration::getNATAddress() const
{
    return myNATAddress;
}


UaConfiguration::UaConfiguration()
        : password("\0"),
        localSipPort("5060"),
        minRtpPort( 1024 ),
        maxRtpPort( 2<<16-1 ),
        sipTransport("UDP"),
        proxyServer("\0"),
        registerOn( false ),
        networkRtpRate( 20 ),          // 20 ms
        provideRingback(false),
        initialDigitTimeout( 16000 ),  // 16 seconds
        interDigitTimeout( 8000 ),     // 8 seconds
        loadGenOn(false),
        callRestartTimer( 17000 ),
        answerRestartTimer( 12000 ),
        rsvpOn(false),
	deviceName("/dev/phone0"),
        xferMode(XferOff),
        myCallIdOn( false ),
        callWaitingOn(false),
        uaTimeout( 5 ),
        subscribeOn( false),
        subscribeExpire( 0 ),
        subscribeTo( "" )
{
}    // UaConfiguration::UaConfiguration


UaConfiguration::~UaConfiguration()
{
}    // UaConfiguration::~UaConfiguration


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
