
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const UaDevice_cxx_Version =
    "$Id: UaDevice.cxx,v 1.28 2002/11/19 20:32:52 sprajpat Exp $";


#include "global.h"
#include "UaDevice.hxx"

#ifdef HAS_PHONEDEV
#include "QuickNetDevice.hxx"
#include "PhoneCardDevice.hxx"
#endif
#ifdef HAS_SOUNDCARD
#include "SoundCardDevice.hxx"
#endif

#include "NullHwDevice.hxx"
#include "UaConfiguration.hxx"
#ifdef HAS_VOICEMAIL
#include "VmcpDevice.hxx"
#endif

#if !defined(WIN32)
#include "SimpleExtDevice.hxx"
#endif

#include "DupCardDevice.hxx"

using namespace Vocal;

///
Sptr < Fifo < Sptr<SipProxyEvent> > > UaDevice::deviceQueue;

///
Sptr<ResGwDevice> UaDevice::uaDevice;


///
Sptr<ResGwDevice>
UaDevice::instance()
{
    assert( uaDevice != 0 );
    return uaDevice;
}  // UaDevice::instance


///
void
UaDevice::instance( const char* useDevice,
                    Sptr < Fifo < Sptr<SipProxyEvent> > > callProcessingQueue )
{
    assert( uaDevice == 0 );

    deviceQueue = new Fifo < Sptr<SipProxyEvent> >;
    assert( deviceQueue != 0 );

#ifdef HAS_PHONEDEV
    if(strcmp(useDevice, "QUICKNET") == 0)
    {
        const char *deviceName =
                   UaConfiguration::instance()->getDeviceName().c_str();
        cpLog( LOG_DEBUG, "Create Quicknet Device %", deviceName );
        uaDevice = new QuickNetDevice( deviceName, deviceQueue,
                                       callProcessingQueue );
        assert( uaDevice != 0 );
    }

    if(strcmp(useDevice, "PHONECARD") == 0)
    {
        const char *deviceName =
                   UaConfiguration::instance()->getDeviceName().c_str();
        cpLog( LOG_DEBUG, "Create Phonecard -pcmcia- Device %", deviceName );
        uaDevice = new PhoneCardDevice( deviceName, deviceQueue,
                                        callProcessingQueue );
        assert( uaDevice != 0 );
    }
#endif

#ifdef HAS_SOUNDCARD
    if(strcmp(useDevice, "SOUNDCARD") == 0)
    {
        const char *deviceName =
                   UaConfiguration::instance()->getDeviceName().c_str();
        cpLog( LOG_DEBUG, "Create Soundcard Device %", deviceName );

        uaDevice = new SoundCardDevice( "/dev/dsp", deviceQueue,
                                        callProcessingQueue );
#ifdef WIN32
        uaDevice = new SoundCardDevice( "/dev/dsp",
                                        deviceQueue,
                                        callProcessingQueue );
#else
//        uaDevice = new DupCardDevice( "/dev/dsp", deviceQueue,
//				      callProcessingQueue );
#endif
        assert( uaDevice != 0 );
    }
#endif

#ifdef HAS_VOICEMAIL
    if(strcmp(useDevice, "VOICEMAIL") == 0)
    {
        cpLog( LOG_DEBUG, "Create Voicemail Device" );
        uaDevice = new VmcpDevice( "vm", 8024, deviceQueue,
                                   callProcessingQueue );
        assert( uaDevice != 0 );
    }
#endif

    if(strcmp(useDevice, "SIMPLE_HW") == 0)
    {
    }

    if(strcmp(useDevice, "NULL") == 0)
    {
        cpLog( LOG_DEBUG, "Create Null Hardware Device" );
        uaDevice = new NullHwDevice( "", deviceQueue,
                                     callProcessingQueue );
        assert( uaDevice != 0 );
    }
} // UaDevice::instance


///
Sptr < Fifo < Sptr<SipProxyEvent> > >
UaDevice::getDeviceQueue()
{
    return deviceQueue;
}    // UaDevice::instance


///
UaDevice::UaDevice()
{}    // UaDevice::UaDevice

///

UaDevice::~UaDevice()
{}    // UaDevice::~UaDevice
