#ifndef UaDeviceEvent_H
#define UaDeviceEvent_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UaDeviceEventVersion =
    "$Id: UaDeviceEvent.hxx,v 1.21 2002/11/09 02:13:08 derekm Exp $";

#include "Sptr.hxx"
#include "SipProxyEvent.hxx"
#include "Fifo.h"
#include "SipCallId.hxx"
#include "DeviceEvent.hxx"

#include "SipTransceiver.hxx"

namespace Vocal
{

//TODO move typedef's into UaDeviceEvent
///
typedef enum
{
    DeviceSignalNULL,
    DeviceSignalDialToneStart,
    DeviceSignalDialToneStop,
    DeviceSignalRingStart,
    DeviceSignalRingStop,
    DeviceSignalRemoteRingbackStart,
    DeviceSignalRemoteRingbackStop,
    DeviceSignalLocalRingbackStart,
    DeviceSignalLocalRingbackStop,
    DeviceSignalBusyStart,
    DeviceSignalBusyStop,
    DeviceSignalFastBusyStart,
    DeviceSignalFastBusyStop,
    DeviceSignalErrorToneStart,
    DeviceSignalErrorToneStop,
    DeviceSignalDigit0,
    DeviceSignalDigit1,
    DeviceSignalDigit2,
    DeviceSignalDigit3,
    DeviceSignalDigit4,
    DeviceSignalDigit5,
    DeviceSignalDigit6,
    DeviceSignalDigit7,
    DeviceSignalDigit8,
    DeviceSignalDigit9,
    DeviceSignalDigitStar,
    DeviceSignalDigitHash,
    DeviceSignalDigitA,
    DeviceSignalDigitB,
    DeviceSignalDigitC,
    DeviceSignalDigitD,
    DeviceSignalFwding,
    DeviceSignalStartCallWaitingBeep,
    DeviceSignalStopCallWaitingBeep
} DeviceSignalType;



/** Events from a device
 *
 */
class UaDeviceEvent : public SipProxyEvent
{
    public:
        ///
        UaDeviceEvent( const Sptr < Fifo < Sptr < SipProxyEvent > > > outputFifo );
        ///
        virtual ~UaDeviceEvent();
        ///
        virtual const char* const name() const;

        ///
        void setSipStack( const Sptr < SipTransceiver > stack );
        ///
        const Sptr < SipTransceiver > getSipStack();

        //TODO move type to protected area
        ///
        DeviceEventType type;
        ///
        Sptr < SipCallId > callId;
	///
	Data text;

    protected:

    private:
        ///
        Sptr < SipTransceiver > sipStack;
};
 
}

#endif
