
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const version =
    "$Id: UaOperator.cxx,v 1.21 2002/11/09 02:13:08 derekm Exp $";



#include "global.h"
#include <SipEvent.hxx>

#include "UaOperator.hxx"
#include "UaTimerEvent.hxx"
#include "UaDigitTimerEvent.hxx"
#include "UaDeviceEvent.hxx"
#include "UaCallInfo.hxx"
#include "UaConfiguration.hxx"
#include "CallContainer.hxx"

using namespace Vocal;

///
UaOperator::UaOperator()
{}


UaOperator::~UaOperator()
{}


///

const char* const
UaOperator::name() const
{
    return "UaOperator";
}

///
bool
UaOperator::setTimer( const Sptr < SipProxyEvent > event, unsigned int timeout )
{
    if ( timeout <= 0 ) return false;

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    Sptr < UaDigitTimerEvent > digitTimerEvent;
    digitTimerEvent.dynamicCast( event );

    if ( sipEvent == 0 && deviceEvent == 0 && digitTimerEvent == 0 )
    {
        cpLog( LOG_DEBUG, "Expecting sipEvent, deviceEvent or digitTimerEvent" );
        assert( 0 );
    } /*if sipEvent  or deviceEvent */

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    Sptr < Fifo < Sptr < SipProxyEvent > > > msgQ;
    Sptr < SipTransceiver > sipStack;
    Sptr < SipCallId > callId;

    if ( sipEvent != 0 )
    {
        msgQ = sipEvent->getFifo();
        sipStack = sipEvent->getSipStack();
        Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
        assert( sipMsg != 0 );
        callId = new SipCallId( sipMsg->getCallId() );
    }
    else if ( deviceEvent != 0 )
    {
        msgQ = deviceEvent->getFifo();
        sipStack = deviceEvent->getSipStack();
        callId = deviceEvent->callId;
    }
    else if (digitTimerEvent !=0 )
    {
        msgQ = digitTimerEvent->getFifo();
        sipStack = digitTimerEvent->getSipStack();
        callId = digitTimerEvent->callId;
    }

    Sptr < UaTimerEvent > timerEvent = new UaTimerEvent( msgQ );
    timerEvent->setCallId(callId);

    cpLog(LOG_DEBUG, "Start TIMER for %d s.", timeout );

    // SAve the timer event in  Call Info
    call->setTimer (timerEvent);
    timerEvent->setCallInfo( call,
                             event->getCallContainer() );
    // Set the sipStack
    timerEvent->setSipStack( sipStack );

    timerEvent->startTimer(timerEvent, timeout*1000);

    return true;
}  //setTimer

///
bool
UaOperator::cancelTimer( const Sptr < SipProxyEvent > event )
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast( event );
    Sptr < UaDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast( event );
    Sptr < UaDigitTimerEvent > digitTimerEvent;
    digitTimerEvent.dynamicCast( event );

    if ( sipEvent == 0 && deviceEvent == 0 && digitTimerEvent == 0 )
    {
        cpLog( LOG_DEBUG, "Expecting sipEvent, deviceEvent or digitTimerEvent." );
        return false;
    } /*if sipEvent  or deviceEvent */

    Sptr < UaCallInfo > call;
    call.dynamicCast( event->getCallInfo() );
    assert( call != 0 );

    if ( call->getTimer() != 0 )
    {
        cpLog( LOG_DEBUG, "Cancelling timer." );
        call->getTimer()->cancelTimer();
        return true;
    }  //getTimer

    cpLog( LOG_ERR, "No timer was found." );
    return false;
}  // cancelTiemr

///
bool
UaOperator::isTimeout( const Sptr < SipProxyEvent > event )
{
    Sptr < UaTimerEvent > timerEvent;
    timerEvent.dynamicCast( event );

    if ( timerEvent != 0 )
    {
        return true;
    }
    else
    {
        return false;
    }
}
