/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ua_cxx_Version =
    "$Id: ua.cxx,v 1.22 2002/04/17 22:18:08 bko Exp $";

#include "global.h"
#include "UaCommandLine.hxx"
#include "Daemon.hxx"
#include "cpLog.h"
#include "UaConfiguration.hxx"
#include "UaBuilder.hxx"
#include "UserAgent.hxx"
#include <stdio.h>

#if defined(HAS_VOICEMAIL)
#include "HeartbeatTxThread.hxx"
#endif

using namespace Vocal;

int
main( int argc, char* argv[] )
{
    cpLogSetLabel( argv[0] );

    UaCommandLine::instance( (const int)argc, (const char**)argv );

    if ( UaCommandLine::instance()->getBoolOpt( "daemon" ) )
    {
        // TODO set cpLog to use syslog

        assert( Daemon() >= 0 );
    }

    cpLogSetPriority( UaCommandLine::instance()->getIntOpt( "loglevel" ) );

    // Turn off retransmission
    if (UaCommandLine::instance( ) -> getIntOpt( "retransmit" ))
    {
        SipTransceiver::reTransOn();
    }
    else
    {
        SipTransceiver::reTransOff();
    }

    if (UaCommandLine::instance()->getIntOpt("retransinitial") != 500 ||
            UaCommandLine::instance()->getIntOpt("retransmax") != 4000 )
    {
        SipTransceiver::setRetransTime(
            UaCommandLine::instance()->getIntOpt("retransinitial"),
            UaCommandLine::instance()->getIntOpt("retransmax")
        );
    }

    cpLog( LOG_NOTICE, "Compiled at " __DATE__ " " __TIME__ );

    const string cfgStr = UaCommandLine::instance()->getStringOpt( "cfgfile" );
    FILE *cfgFile = fopen( cfgStr.c_str(), "r");
    if ( cfgFile == 0 )
    {
        cerr << "can not open " << cfgStr << endl;
        cerr << "Usage: " << argv[0] << " " << appUsage << endl;
        exit( 0 );

    }
    else
    {
        fclose( cfgFile );
        UaConfiguration::instance( cfgStr );
    }

    // if the config file has a log level, do something about it
    if(UaConfiguration::instance()->getLogFilename() != "")
    {
	int retval = cpLogOpen(
	    UaConfiguration::instance()->getLogFilename().c_str());

	if(retval == 0)
	{
	    cpLog(LOG_ALERT, "Could not open %s", 
		  UaConfiguration::instance()->getLogFilename().c_str());
	}
    }

    Sptr < UaBuilder > uaBuilder = new UaBuilder;

    UserAgent ua( uaBuilder, Data( UaConfiguration::instance()->getLocalSipPort() ).convertInt() );

    ua.run();

#if defined(HAS_VOICEMAIL)
    if ( UaCommandLine::instance()->getBoolOpt( "voicemail" ) )
    {
        cpLog(LOG_DEBUG, " Inside voicemail");
        //create heartbeat transmit thread and start it.
        int sipPort = atoi(UaConfiguration::instance()->getLocalSipPort().c_str());
        cpLog(LOG_DEBUG, " Local Sip port = %d" , sipPort);

        if(!UaCommandLine::instance()->getBoolOpt("no_heartbeat"))
        {
            Sptr < HeartbeatTxThread > heartbeatThread 
                = new HeartbeatTxThread(sipPort,
                                        500,
                                        (const char*)"226.2.2.5",
                                        6000);
            heartbeatThread->run();
            heartbeatThread->join();
        }
    }
#endif    

    ua.join();

    return 0;
}    // ua main()


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
