#ifndef TcpClientSocket_hxx
#define TcpClientSocket_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TcpClientSocketHeaderVersion =
    "$Id: Tcp_ClientSocket.hxx,v 1.5.2.1 2003/01/30 22:39:10 bko Exp $";

//User define class
#include "Connection.hxx"
#include "VNetworkException.hxx"
#include "Data.hxx"

class NetworkAddress;

/**
   Create A client socket for TCP.<p>

   <b>Usage:</b>

   <pre>
   NetworkAddress serverAddress;
   TcpClientSocket mySocket(serverAddress, true, false);  // nonblocking

   mySocket.connect(); // connect

   Connection& conn = mySocket.getConn();

   // now, you can use connection to read or write to the server.  see
   // Connection class for details.

   </pre>
*/
class TcpClientSocket
{
    public:

        /**
           Create client TCP connection.

           @param hostName   host:port to connect to.  Host can be fqdn or ip addr as string.
           @param closeCon   allow close() to close the connection.
           @param blocking   reads should be blocking.

        */
        TcpClientSocket(const string& hostName,
                        bool closeCon = true, bool blocking = true);

        /**
           Create client TCP connection.

           @param hostName   host name to connect to.  Host can be fqdn or ip addr as string.
           @param port       host port to connect to.
           @param closeCon   allow close() to close the connection.
           @param blocking   reads should be blocking.

        */
        TcpClientSocket(const string& hostName, int servPort,
                        bool closeCon = true, bool blocking = true);

        /**
           Create client TCP connection.

           @param server    host and port to connect to.
           @param closeCon   allow close() to close the connection.
           @param blocking   reads should be blocking.

        */
        TcpClientSocket(const NetworkAddress& server,
                        bool closeCon = true, bool blocking = true);

        /**
           Create client TCP connection.

           This is a copy constructor.
        */
        TcpClientSocket(const TcpClientSocket&);

        /// assignment operator
        TcpClientSocket& operator=(TcpClientSocket& other);


        ~TcpClientSocket();


        /**
           connect to the far side.
        */
        void connect() throw (VNetworkException&);
        /**
           close the connnection.
        */
        void close();

        /**
           get the Connection which was created by this object.
        */
        inline Connection& getConn()
        {
            return _conn;
        };
    private:
        void initalize();
        const char* connectionDesc(struct addrinfo* laddr, char* descBuf, int bufLen) const;


        Connection _conn;
        Data _hostName;
        int _serverPort;
        bool _closeCon;
        bool _blocking;
};

/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */

#endif
